@@ -1,9 +1,9 @@
 /*
- * "$Id: aix.c,v 1.12 2002/10/17 18:13:13 mike Exp $"
+ * "$Id: aix.c 670 2006-04-05 15:08:08Z mike $"
  *
  *   AIX package gateway for the ESP Package Manager (EPM).
  *
- *   Copyright 1999-2002 by Easy Software Products.
+ *   Copyright 1999-2006 by Easy Software Products.
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -18,6 +18,8 @@
  * Contents:
  *
  *   make_aix()     - Make an AIX software distribution package.
+ *   aix_addfile()  - Add a file to the AIX directory list...
+ *   aix_fileset()  - Write a subpackage description...
  *   aix_version()  - Generate an AIX version number.
  *   write_liblpp() - Create the liblpp.a file for the root or /usr parts.
  */
@@ -29,14 +31,30 @@
 #include "epm.h"
 
 
+/*
+ * Directory information...
+ */
+
+typedef struct
+{
+  char	dst[1024];			/* Output directory */
+  int	blocks;				/* Size of files in this directory */
+} aixdir_t;
+
+
 /*
  * Local functions...
  */
 
+static int	aix_addfile(int type, const char *src, const char *dst,
+		            int num_dirs, aixdir_t **dirs);
+static void	aix_fileset(FILE *fp, const char *prodname, dist_t *dist,
+		            const char *subpackage);
 static char	*aix_version(const char *version);
 static int	write_liblpp(const char *prodname,
 		             const char *directory,
-		             dist_t *dist, int root);
+		             dist_t *dist, int root,
+			     const char *subpackage);
 
 
 /*
@@ -51,7 +69,8 @@ static const char	*files[] =	/* Control files... */
 			  "inventory",
 			  "post_i",
 			  "pre_i",
-			  "unpost_i"
+			  "unpost_i",
+                          "unpre_i"
 			};
 
 
@@ -68,12 +87,9 @@ make_aix(const char     *prodname,	/* I - Product short name */
 {
   int			i;		/* Looping var */
   FILE			*fp;		/* Control file */
-  char			name[1024];	/* Full product name */
-  char			filename[1024];	/* Destination filename */
-  char			current[1024];	/* Current directory */
-  int			blocks;		/* Number of blocks needed */
-  struct stat		fileinfo;	/* File information */
-  depend_t		*d;		/* Current dependency */
+  char			name[1024],	/* Full product name */
+			filename[1024],	/* Destination filename */
+			current[1024];	/* Current directory */
   file_t		*file;		/* Current distribution file */
   struct passwd		*pwd;		/* Pointer to user record */
   struct group		*grp;		/* Pointer to group record */
@@ -85,13 +101,14 @@ make_aix(const char     *prodname,	/* I - Product short name */
   if (Verbosity)
     puts("Creating AIX distribution...");
 
-  if (dist->relnumber)
+  if (dist->release[0])
   {
     if (platname[0])
-      snprintf(name, sizeof(name), "%s-%s-%d-%s", prodname, dist->version, dist->relnumber,
-              platname);
+      snprintf(name, sizeof(name), "%s-%s-%s-%s", prodname, dist->version,
+               dist->release, platname);
     else
-      snprintf(name, sizeof(name), "%s-%s-%d", prodname, dist->version, dist->relnumber);
+      snprintf(name, sizeof(name), "%s-%s-%s", prodname, dist->version,
+               dist->release);
   }
   else if (platname[0])
     snprintf(name, sizeof(name), "%s-%s-%s", prodname, dist->version, platname);
@@ -119,72 +136,9 @@ make_aix(const char     *prodname,	/* I - Product short name */
     return (1);
   }
 
-  fprintf(fp, "4 R I %s {\n", prodname);
-  fprintf(fp, "%s %s 01 N B x %s\n", prodname,
-          aix_version(dist->version), dist->product);
-
- /*
-  * Dependencies...
-  */
-
-  fputs("[\n", fp);
-  for (i = dist->num_depends, d = dist->depends; i > 0; i --, d ++)
-    if (d->type == DEPEND_REQUIRES)
-      fprintf(fp, "*prereq %s %s\n", d->product, aix_version(d->version[0]));
-
- /*
-  * Installation sizes...
-  */
-
-  fputs("%\n", fp);
-  for (i = dist->num_files, file = dist->files, blocks = 0; i > 0; i --, file ++)
-    if (strncmp(file->dst, "/usr/", 5) != 0)
-      switch (tolower(file->type))
-      {
-        case 'd' :
-	case 'l' :
-	    blocks ++;
-	    break;
-
-	default :
-	    if (!stat(file->src, &fileinfo))
-	      blocks += (fileinfo.st_size + 511) / 512;
-	    break;
-      }
-  fprintf(fp, "/ %d\n", blocks);
-
-  for (i = dist->num_files, file = dist->files, blocks = 0; i > 0; i --, file ++)
-    if (strncmp(file->dst, "/usr/", 5) == 0)
-      switch (tolower(file->type))
-      {
-        case 'd' :
-	case 'l' :
-	    blocks ++;
-	    break;
-
-	default :
-	    if (!stat(file->src, &fileinfo))
-	      blocks += (fileinfo.st_size + 511) / 512;
-	    break;
-      }
-  fprintf(fp, "/usr %d\n", blocks);
-
- /*
-  * This package supercedes which others?
-  */
-
-  fputs("%\n", fp);
-  for (i = dist->num_depends, d = dist->depends; i > 0; i --, d ++)
-    if (d->type == DEPEND_REPLACES)
-      fprintf(fp, "%s %s", d->product, aix_version(d->version[0]));
-
- /*
-  * Fix information is only used for updates (patches)...
-  */
-
-  fputs("%\n", fp);
-  fputs("]\n", fp);
-  fputs("}\n", fp);
+  aix_fileset(fp, prodname, dist, NULL);
+  for (i = 0; i < dist->num_subpackages; i ++)
+    aix_fileset(fp, prodname, dist, dist->subpackages[i]);
 
   fclose(fp);
 
@@ -192,13 +146,17 @@ make_aix(const char     *prodname,	/* I - Product short name */
   * Write the root partition liblpp.a file...
   */
 
-  write_liblpp(prodname, directory, dist, 1);
+  write_liblpp(prodname, directory, dist, 1, NULL);
+  for (i = 0; i < dist->num_subpackages; i ++)
+    write_liblpp(prodname, directory, dist, 1, dist->subpackages[i]);
 
  /*
   * Write the usr partition liblpp.a file...
   */
 
-  write_liblpp(prodname, directory, dist, 0);
+  write_liblpp(prodname, directory, dist, 0, NULL);
+  for (i = 0; i < dist->num_subpackages; i ++)
+    write_liblpp(prodname, directory, dist, 0, dist->subpackages[i]);
 
  /*
   * Copy the files over...
@@ -227,7 +185,8 @@ make_aix(const char     *prodname,	/* I - Product short name */
     {
       case 'c' :
       case 'f' :
-          if (strncmp(file->dst, "/usr/", 5) == 0)
+          if (!strncmp(file->dst, "/usr/", 5) ||
+	      !strncmp(file->dst, "/opt/", 5))
             snprintf(filename, sizeof(filename), "%s/%s%s", directory, prodname,
 	             file->dst);
 	  else
@@ -243,12 +202,25 @@ make_aix(const char     *prodname,	/* I - Product short name */
           break;
       case 'i' :
           for (runlevels = get_runlevels(file, "2");
-	       isdigit(*runlevels);
+	       isdigit(*runlevels & 255);
 	       runlevels ++)
 	  {
             snprintf(filename, sizeof(filename),
-	             "%s/%s/usr/lpp/%s/inst_root/etc/rc.d/rc%c.d/%s",
-	             directory, prodname, prodname, *runlevels, file->dst);
+	             "%s/%s/usr/lpp/%s/inst_root/etc/rc.d/rc%c.d/S%02d%s",
+	             directory, prodname, prodname, *runlevels,
+		     get_start(file, 99), file->dst);
+
+	    if (Verbosity > 1)
+	      printf("%s -> %s...\n", file->src, filename);
+
+	    if (copy_file(filename, file->src, file->mode,
+	                  pwd ? pwd->pw_uid : 0, grp ? grp->gr_gid : 0))
+	      return (1);
+
+            snprintf(filename, sizeof(filename),
+	             "%s/%s/usr/lpp/%s/inst_root/etc/rc.d/rc%c.d/K%02d%s",
+	             directory, prodname, prodname, *runlevels,
+		     get_stop(file, 0), file->dst);
 
 	    if (Verbosity > 1)
 	      printf("%s -> %s...\n", file->src, filename);
@@ -259,7 +231,10 @@ make_aix(const char     *prodname,	/* I - Product short name */
 	  }
           break;
       case 'd' :
-          if (strncmp(file->dst, "/usr/", 5) == 0)
+          if (!strcmp(file->dst, "/usr") ||
+	      !strncmp(file->dst, "/usr/", 5) ||
+	      !strcmp(file->dst, "/opt") ||
+	      !strncmp(file->dst, "/opt/", 5))
             snprintf(filename, sizeof(filename), "%s/%s%s", directory, prodname,
 	             file->dst);
 	  else
@@ -273,7 +248,8 @@ make_aix(const char     *prodname,	/* I - Product short name */
 			 grp ? grp->gr_gid : 0);
           break;
       case 'l' :
-          if (strncmp(file->dst, "/usr/", 5) == 0)
+          if (!strncmp(file->dst, "/usr/", 5) ||
+	      !strncmp(file->dst, "/opt/", 5))
             snprintf(filename, sizeof(filename), "%s/%s%s", directory, prodname,
 	             file->dst);
 	  else
@@ -297,8 +273,9 @@ make_aix(const char     *prodname,	/* I - Product short name */
 
   snprintf(filename, sizeof(filename), "%s/%s", directory, prodname);
 
-  if (run_command(filename, "sh -c \'find . -print | backup -i -f ../%s.bff -q %s\'",
-                  prodname, Verbosity ? "-v" : "", prodname))
+  if (run_command(filename,
+		  "sh -c \'find . -print | backup -i -f ../%s.bff -q %s\'",
+                  prodname, Verbosity ? "-v" : ""))
     return (1);
 
  /*
@@ -326,6 +303,171 @@ make_aix(const char     *prodname,	/* I - Product short name */
 }
 
 
+/*
+ * 'aix_addfile()' - Add a file to the AIX directory list...
+ */
+
+static int				/* O  - New number dirs */
+aix_addfile(int        type,		/* I  - Filetype */
+            const char *src,		/* I  - Source path */
+            const char *dst,		/* I  - Destination path */
+	    int        num_dirs,	/* I  - Number of directories */
+            aixdir_t   **dirs)		/* IO - Directories */
+{
+  int		i, j;			/* Looping vars */
+  int		blocks;			/* Blocks to add... */
+  struct stat	fileinfo;		/* File information */
+  aixdir_t	*temp;			/* Temporary pointer */
+  char		dstpath[1024],		/* Destination path */
+		*dstptr;		/* Pointer into destination */
+
+
+ /*
+  * Determine the destination path and block size...
+  */
+
+  strlcpy(dstpath, dst, sizeof(dstpath));
+
+  if (type == 'd')
+  {
+    blocks = 1;
+    dstptr = dstpath + strlen(dstpath) - 1;
+  }
+  else
+  {
+    dstptr = strrchr(dstpath, '/');
+
+    if (type == 'l')
+      blocks = 1;
+    else if (!stat(src, &fileinfo))
+      blocks = (fileinfo.st_size + 511) / 512;
+    else
+      blocks = 0;
+  }
+
+  if (dstptr && *dstptr == '/' && dstptr > dstpath)
+    *dstptr = '\0';
+
+ /*
+  * Now see if the destination path is in the array...
+  */
+
+  temp = *dirs;
+
+  for (i = 0; i < num_dirs; i ++)
+    if ((j = strcmp(temp[i].dst, dstpath)) == 0)
+    {
+      temp[i].blocks += blocks;
+      return (num_dirs);
+    }
+    else if (j > 0)
+      break;
+
+ /*
+  * Not in the list; allocate a new one...
+  */
+
+  if (num_dirs == 0)
+    temp = malloc(sizeof(aixdir_t));
+  else
+    temp = realloc(*dirs, (num_dirs + 1) * sizeof(aixdir_t));
+
+  if (!temp)
+    return (num_dirs);
+
+  *dirs = temp;
+  temp  += i;
+
+  if (i < num_dirs)
+    memmove(temp + 1, temp, (num_dirs - i) * sizeof(aixdir_t));
+
+  strcpy(temp->dst, dstpath);
+  temp->blocks = blocks;
+
+  return (num_dirs + 1);
+}
+
+
+/*
+ * 'aix_fileset()' - Write a subpackage description...
+ */
+
+static void
+aix_fileset(FILE       *fp,		/* I - File to write to */
+            const char *prodname,	/* I - Product name */
+            dist_t     *dist,		/* I - Distribution */
+	    const char *subpackage)	/* I - Subpackage */
+{
+  int			i;		/* Looping var */
+  depend_t		*d;		/* Current dependency */
+  file_t		*file;		/* Current distribution file */
+  int			num_dirs;	/* Number of directories */
+  aixdir_t		*dirs;		/* Directories */
+
+
+ /*
+  * Start fileset definition...
+  */
+
+  fprintf(fp, "4 R I %s {\n", prodname);
+
+  if (subpackage)
+    fprintf(fp, "%s.%s", prodname, subpackage);
+  else
+    fprintf(fp, "%s", prodname);
+
+  fprintf(fp, " %s 01 N B x %s\n", aix_version(dist->version), dist->product);
+
+ /*
+  * Dependencies...
+  */
+
+  fputs("[\n", fp);
+  for (i = dist->num_depends, d = dist->depends; i > 0; i --, d ++)
+    if (d->type == DEPEND_REQUIRES && d->subpackage == subpackage &&
+        strcmp(d->product, "_self"))
+      fprintf(fp, "*prereq %s %s\n", d->product, aix_version(d->version[0]));
+
+ /*
+  * Installation sizes...
+  */
+
+  fputs("%\n", fp);
+
+  num_dirs = 0;
+  dirs     = NULL;
+
+  for (i = dist->num_files, file = dist->files; i > 0; i --, file ++)
+    if (file->subpackage == subpackage)
+      num_dirs = aix_addfile(tolower(file->type), file->src, file->dst,
+                             num_dirs, &dirs);
+
+  for (i = 0; i < num_dirs; i ++)
+    fprintf(fp, "%s %d\n", dirs[i].dst, dirs[i].blocks);
+
+  if (num_dirs > 0)
+    free(dirs);
+
+ /*
+  * This package supercedes which others?
+  */
+
+  fputs("%\n", fp);
+  for (i = dist->num_depends, d = dist->depends; i > 0; i --, d ++)
+    if (d->type == DEPEND_REPLACES && d->subpackage == subpackage &&
+        strcmp(d->product, "_self"))
+      fprintf(fp, "%s %s", d->product, aix_version(d->version[0]));
+
+ /*
+  * Fix information is only used for updates (patches)...
+  */
+
+  fputs("%\n", fp);
+  fputs("]\n", fp);
+  fputs("}\n", fp);
+}
+
+
 /*
  * 'aix_version()' - Generate an AIX version number.
  */
@@ -359,23 +501,33 @@ static int				/* O - 0 = success, 1 = fail */
 write_liblpp(const char     *prodname,	/* I - Product short name */
              const char     *directory,	/* I - Directory for distribution files */
              dist_t         *dist,	/* I - Distribution information */
-             int            root)	/* I - Root partition? */
+             int            root,	/* I - Root partition? */
+	     const char     *subpackage)/* I - Subpackage */
 {
   int			i;		/* Looping var */
   FILE			*fp;		/* Control file */
-  char			filename[1024];	/* Destination filename */
+  char			filename[1024],	/* Destination filename */
+			prodfull[1024];	/* Full product name */
   struct stat		fileinfo;	/* File information */
   command_t		*c;		/* Current command */
   file_t		*file;		/* Current distribution file */
+  int			configcount;	/* Number of config files */
+  int 			shared_file;	/* Shared file? */
+  const char		*runlevels;	/* Run levels */
 
 
  /*
   * Progress info...
   */
 
+  if (subpackage)
+    snprintf(prodfull, sizeof(prodfull), "%s.%s", prodname, subpackage);
+  else
+    strlcpy(prodfull, prodname, sizeof(prodfull));
+
   if (Verbosity)
-    puts(root ? "Creating root partition liblpp.a file..." :
-                "Creating /usr partition liblpp.a file...");
+    printf("Updating %s partition liblpp.a file for %s...\n",
+           root ? "root" : "shared", prodfull);
 
  /*
   * Write the product.al file for installp...
@@ -384,7 +536,7 @@ write_liblpp(const char     *prodname,	/* I - Product short name */
   if (Verbosity > 1)
     puts("    Creating .al file...");
 
-  snprintf(filename, sizeof(filename), "%s/%s.al", directory, prodname);
+  snprintf(filename, sizeof(filename), "%s/%s.al", directory, prodfull);
 
   if ((fp = fopen(filename, "w")) == NULL)
   {
@@ -394,17 +546,52 @@ write_liblpp(const char     *prodname,	/* I - Product short name */
   }
 
   for (i = dist->num_files, file = dist->files; i > 0; i --, file ++)
-    switch (tolower(file->type))
-    {
-      case 'i' :
-          if (root)
-            qprintf(fp, "./etc/rc.d/rc2.d/S99%s\n", file->dst);
-	  break;
-      default :
-          if ((strncmp(file->dst, "/usr/", 5) != 0) == root)
-            qprintf(fp, ".%s\n", file->dst);
-	  break;
-    }
+    if (file->subpackage == subpackage)
+      switch (tolower(file->type))
+      {
+	case 'i' :
+            for (runlevels = get_runlevels(file, "2");
+		 isdigit(*runlevels & 255);
+		 runlevels ++)
+            {
+	      if (root)
+	        putc('.', fp);
+	      else
+        	qprintf(fp, "./usr/lpp/%s/inst_root", prodfull);
+
+              if (*runlevels == '0')
+        	qprintf(fp, "/etc/rc.d/rc0.d/K%02d%s\n",
+		        get_stop(file, 0), file->dst);
+	      else
+        	qprintf(fp, "/etc/rc.d/rc%c.d/S%02d%s\n",
+	        	*runlevels, get_start(file, 99), file->dst);
+            }
+	    break;
+
+	default :
+            shared_file = !(strcmp(file->dst, "/usr") && 
+                            strncmp(file->dst, "/usr/", 5) && 
+                            strcmp(file->dst, "/opt") && 
+                            strncmp(file->dst, "/opt/", 5));
+
+           /*
+	    * Put file in root or share .al file as appropriate
+            */
+
+            if ((shared_file && !root) || (!shared_file && root))
+              qprintf(fp, ".%s\n", file->dst);
+
+           /*
+	    * Put any root file in the share .al so it will be extracted
+	    * to /usr/lpp/<prodfull>/inst_root directory.  I have no
+	    * idea if this is really the way to do it but it seems to
+	    * work...
+	    */
+
+            if (!shared_file && !root)
+              qprintf(fp, "./usr/lpp/%s/inst_root%s\n", prodfull, file->dst);
+	    break;
+      }
 
   fclose(fp);
 
@@ -415,20 +602,26 @@ write_liblpp(const char     *prodname,	/* I - Product short name */
   if (Verbosity > 1)
     puts("    Creating .cfgfiles file...");
 
-  snprintf(filename, sizeof(filename), "%s/%s.cfgfiles", directory, prodname);
+  snprintf(filename, sizeof(filename), "%s/%s.cfgfiles", directory, prodfull);
 
   if ((fp = fopen(filename, "w")) == NULL)
   {
-    fprintf(stderr, "epm: Unable to create .cfgfiles file \"%s\" - %s\n", filename,
-            strerror(errno));
+    fprintf(stderr, "epm: Unable to create .cfgfiles file \"%s\" - %s\n",
+            filename, strerror(errno));
     return (1);
   }
 
+  configcount = 0;
+
   for (i = dist->num_files, file = dist->files; i > 0; i --, file ++)
-    if (tolower(file->type) == 'c' &&
-        (strncmp(file->dst, "/usr/", 5) != 0) == root)
+    if (tolower(file->type) == 'c' && file->subpackage == subpackage &&
+        (strcmp(file->dst, "/usr") ||
+	 strncmp(file->dst, "/usr/", 5) ||
+	 strcmp(file->dst, "/opt") ||
+	 strncmp(file->dst, "/opt/", 5)) == root)
     {
       qprintf(fp, ".%s hold_new\n", file->dst);
+      configcount ++;
     }
 
   fclose(fp);
@@ -440,7 +633,7 @@ write_liblpp(const char     *prodname,	/* I - Product short name */
   if (Verbosity > 1)
     puts("    Creating .copyright file...");
 
-  snprintf(filename, sizeof(filename), "%s/%s.copyright", directory, prodname);
+  snprintf(filename, sizeof(filename), "%s/%s.copyright", directory, prodfull);
 
   if ((fp = fopen(filename, "w")) == NULL)
   {
@@ -463,12 +656,12 @@ write_liblpp(const char     *prodname,	/* I - Product short name */
     if (Verbosity > 1)
       puts("    Creating .pre_i file...");
 
-    snprintf(filename, sizeof(filename), "%s/%s.pre_i", directory, prodname);
+    snprintf(filename, sizeof(filename), "%s/%s.pre_i", directory, prodfull);
 
     if ((fp = fopen(filename, "w")) == NULL)
     {
-      fprintf(stderr, "epm: Unable to create .pre_i file \"%s\" - %s\n", filename,
-              strerror(errno));
+      fprintf(stderr, "epm: Unable to create .pre_i file \"%s\" - %s\n",
+              filename, strerror(errno));
       return (1);
     }
 
@@ -478,7 +671,7 @@ write_liblpp(const char     *prodname,	/* I - Product short name */
     fputs("# " EPM_VERSION "\n", fp);
 
     for (c = dist->commands, i = dist->num_commands; i > 0; i --, c ++)
-      if (c->type == COMMAND_PRE_INSTALL)
+      if (c->type == COMMAND_PRE_INSTALL && c->subpackage == subpackage)
 	fprintf(fp, "%s\n", c->command);
 
     fclose(fp);
@@ -490,12 +683,12 @@ write_liblpp(const char     *prodname,	/* I - Product short name */
     if (Verbosity > 1)
       puts("    Creating .post_i file...");
 
-    snprintf(filename, sizeof(filename), "%s/%s.post_i", directory, prodname);
+    snprintf(filename, sizeof(filename), "%s/%s.post_i", directory, prodfull);
 
     if ((fp = fopen(filename, "w")) == NULL)
     {
-      fprintf(stderr, "epm: Unable to create .post_i file \"%s\" - %s\n", filename,
-              strerror(errno));
+      fprintf(stderr, "epm: Unable to create .post_i file \"%s\" - %s\n",
+              filename, strerror(errno));
       return (1);
     }
 
@@ -505,24 +698,25 @@ write_liblpp(const char     *prodname,	/* I - Product short name */
     fputs("# " EPM_VERSION "\n", fp);
 
     for (c = dist->commands, i = dist->num_commands; i > 0; i --, c ++)
-      if (c->type == COMMAND_POST_INSTALL)
+      if (c->type == COMMAND_POST_INSTALL && c->subpackage == subpackage)
 	fprintf(fp, "%s\n", c->command);
 
     fclose(fp);
 
    /*
-    * Write the product.unpost_i file for installp...
+    * Write the product.unpre_i file for installp (note, despite the name,
+    * unpre_i is the post-removal script...)
     */
 
     if (Verbosity > 1)
-      puts("    Creating .unpost_i file...");
+      puts("	Creating .unpre_i file...");
 
-    snprintf(filename, sizeof(filename), "%s/%s.unpost_i", directory, prodname);
+    snprintf(filename, sizeof(filename), "%s/%s.unpre_i", directory, prodfull);
 
     if ((fp = fopen(filename, "w")) == NULL)
     {
-      fprintf(stderr, "epm: Unable to create .unpost_i file \"%s\" - %s\n", filename,
-              strerror(errno));
+      fprintf(stderr, "epm: Unable to create .unpre_i file \"%s\" - %s\n",
+              filename, strerror(errno));
       return (1);
     }
 
@@ -532,7 +726,35 @@ write_liblpp(const char     *prodname,	/* I - Product short name */
     fputs("# " EPM_VERSION "\n", fp);
 
     for (c = dist->commands, i = dist->num_commands; i > 0; i --, c ++)
-      if (c->type == COMMAND_PRE_REMOVE)
+      if (c->type == COMMAND_POST_REMOVE && c->subpackage == subpackage)
+	fprintf(fp, "%s\n", c->command);
+
+    fclose(fp);
+
+   /*
+    * Write the product.unpost_i file for installp (note, despite the name,
+    * unpost_i is the pre-removal script...)
+    */
+
+    if (Verbosity > 1)
+      puts("    Creating .unpost_i file...");
+
+    snprintf(filename, sizeof(filename), "%s/%s.unpost_i", directory, prodfull);
+
+    if ((fp = fopen(filename, "w")) == NULL)
+    {
+      fprintf(stderr, "epm: Unable to create .unpost_i file \"%s\" - %s\n",
+              filename, strerror(errno));
+      return (1);
+    }
+
+    fchmod(fileno(fp), 0755);
+
+    fputs("#!/bin/sh\n", fp);
+    fputs("# " EPM_VERSION "\n", fp);
+
+    for (c = dist->commands, i = dist->num_commands; i > 0; i --, c ++)
+     if (c->type == COMMAND_PRE_REMOVE && c->subpackage == subpackage)
 	fprintf(fp, "%s\n", c->command);
 
     fclose(fp);
@@ -545,60 +767,94 @@ write_liblpp(const char     *prodname,	/* I - Product short name */
   if (Verbosity > 1)
     puts("    Creating .inventory file...");
 
-  snprintf(filename, sizeof(filename), "%s/%s.inventory", directory, prodname);
+  snprintf(filename, sizeof(filename), "%s/%s.inventory", directory, prodfull);
 
   if ((fp = fopen(filename, "w")) == NULL)
   {
-    fprintf(stderr, "epm: Unable to create .inventory file \"%s\" - %s\n", filename,
-            strerror(errno));
+    fprintf(stderr, "epm: Unable to create .inventory file \"%s\" - %s\n",
+            filename, strerror(errno));
     return (1);
   }
 
   for (i = dist->num_files, file = dist->files; i > 0; i --, file ++)
   {
+    if (file->subpackage != subpackage)
+      continue;
+
     if (root)
     {
-      if (strncmp(file->dst, "/usr/", 5) == 0)
+      if (!strcmp(file->dst, "/usr") ||
+	  !strncmp(file->dst, "/usr/", 5) ||
+	  !strcmp(file->dst, "/opt") ||
+	  !strncmp(file->dst, "/opt/", 5))
         continue;
     }
     else
     {
       if (tolower(file->type) == 'c' ||
-          strncmp(file->dst, "/usr/", 5) != 0)
+          (strcmp(file->dst, "/usr") &&
+	   strncmp(file->dst, "/usr/", 5) &&
+	   strcmp(file->dst, "/opt") &&
+	   strncmp(file->dst, "/opt/", 5)))
 	continue;
     }
 
     switch (tolower(file->type))
     {
       case 'i' :
-          qprintf(fp, "/etc/rc.d/rc2.d/S99%s:\n", file->dst);
+          for (runlevels = get_runlevels(file, "2");
+	       isdigit(*runlevels & 255);
+	       runlevels ++)
+          {
+	    if (*runlevels == '0')
+	      qprintf(fp, "/etc/rc.d/rc0.d/K%02d%s:\n",
+	              get_stop(file, 0), file->dst);
+            else
+	      qprintf(fp, "/etc/rc.d/rc%c.d/S%02d%s:\n",
+	              *runlevels, get_start(file, 99), file->dst);
+
+	    fprintf(fp, "    class=apply,inventory,%s\n", prodfull);
+
+            fputs("    type=FILE\n", fp);
+            if (!stat(file->src, &fileinfo))
+	      fprintf(fp, "    size=%d\n", (int)fileinfo.st_size);
+
+	    fprintf(fp, "    owner=%s\n", file->user);
+	    fprintf(fp, "    group=%s\n", file->group);
+	    fprintf(fp, "    mode=%04o\n", (unsigned)file->mode);
+	    fputs("\n", fp);
+	  }
 	  break;
       default :
           qprintf(fp, "%s:\n", file->dst);
+	  fprintf(fp, "    class=apply,inventory,%s\n", prodfull);
+
+	  switch (tolower(file->type))
+	  {
+	    case 'd' :
+        	fputs("    type=DIRECTORY\n", fp);
+        	break;
+	    case 'l' :
+        	fputs("    type=SYMLINK\n", fp);
+		qprintf(fp, "    target=%s\n", file->src);
+        	break;
+	    case 'c' :
+        	fputs("    type=FILE\n", fp);
+        	fputs("    size=VOLATILE\n", fp);
+		break;
+	    default :
+        	fputs("    type=FILE\n", fp);
+        	if (!stat(file->src, &fileinfo))
+		  fprintf(fp, "    size=%ld\n", (long)fileinfo.st_size);
+		break;
+	  }
+
+	  fprintf(fp, "    owner=%s\n", file->user);
+	  fprintf(fp, "    group=%s\n", file->group);
+	  fprintf(fp, "    mode=%04o\n", (unsigned)file->mode);
+	  fputs("\n", fp);
 	  break;
     }
-
-    fprintf(fp, "    class=apply,inventory,%s\n", prodname);
-
-    switch (tolower(file->type))
-    {
-      case 'd' :
-          fputs("    type=DIRECTORY\n", fp);
-          break;
-      case 'l' :
-          fputs("    type=SYMLINK\n", fp);
-	  qprintf(fp, "    target=%s\n", file->src);
-          break;
-      default :
-          fputs("    type=FILE\n", fp);
-          if (!stat(file->src, &fileinfo))
-	    fprintf(fp, "    size=%d\n", (int)fileinfo.st_size);
-	  break;
-    }
-    fprintf(fp, "    owner=%s\n", file->user);
-    fprintf(fp, "    group=%s\n", file->group);
-    fprintf(fp, "    mode=%04o\n", file->mode);
-    fputs("\n", fp);
   }
 
   fclose(fp);
@@ -644,6 +900,9 @@ write_liblpp(const char     *prodname,	/* I - Product short name */
     if (i >= 4 && !root)
       break;
 
+    if (i == 1 && !configcount)
+      continue;
+
     if (run_command(directory, "ar rc %s %s.%s",
                     filename, prodname, files[i]))
       return (1);
@@ -652,6 +911,7 @@ write_liblpp(const char     *prodname,	/* I - Product short name */
   return (0);
 }
 
+
 /*
- * End of "$Id: aix.c,v 1.12 2002/10/17 18:13:13 mike Exp $".
+ * End of "$Id: aix.c 670 2006-04-05 15:08:08Z mike $".
  */
