@@ -1,9 +1,9 @@
 /*
- * "$Id: deb.c,v 1.19 2002/10/17 14:56:13 mike Exp $"
+ * "$Id: deb.c 670 2006-04-05 15:08:08Z mike $"
  *
  *   Debian package gateway for the ESP Package Manager (EPM).
  *
- *   Copyright 1999-2002 by Easy Software Products.
+ *   Copyright 1999-2006 by Easy Software Products.
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -17,7 +17,8 @@
  *
  * Contents:
  *
- *   make_deb() - Make a Debian software distribution package.
+ *   make_deb()        - Make a Debian software distribution package.
+ *   make_subpackage() - Make a subpackage...
  */
 
 /*
@@ -27,6 +28,16 @@
 #include "epm.h"
 
 
+/*
+ * Local functions...
+ */
+
+static int	make_subpackage(const char *prodname, const char *directory,
+		                const char *platname, dist_t *dist,
+		                struct utsname *platform,
+				const char *subpackage);
+
+
 /*
  * 'make_deb()' - Make a Debian software distribution package.
  */
@@ -37,18 +48,109 @@ make_deb(const char     *prodname,	/* I - Product short name */
          const char     *platname,	/* I - Platform name */
          dist_t         *dist,		/* I - Distribution information */
 	 struct utsname *platform)	/* I - Platform information */
+{
+  int		i;			/* Looping var */
+  tarf_t	*tarfile;		/* Distribution tar file */
+  char		name[1024],		/* Full product name */
+		filename[1024];		/* File to archive */
+
+
+  if (make_subpackage(prodname, directory, platname, dist, platform, NULL))
+    return (1);
+
+  for (i = 0; i < dist->num_subpackages; i ++)
+    if (make_subpackage(prodname, directory, platname, dist, platform,
+                        dist->subpackages[i]))
+      return (1);
+
+ /*
+  * Build a compressed tar file to hold all of the subpackages...
+  */
+
+  if (dist->num_subpackages)
+  {
+   /*
+    * Figure out the full name of the distribution...
+    */
+
+    if (dist->release[0])
+      snprintf(name, sizeof(name), "%s-%s-%s", prodname, dist->version,
+               dist->release);
+    else
+      snprintf(name, sizeof(name), "%s-%s", prodname, dist->version);
+
+    if (platname[0])
+    {
+      strlcat(name, "-", sizeof(name));
+      strlcat(name, platname, sizeof(name));
+    }
+
+   /*
+    * Create a compressed tar file...
+    */
+
+    snprintf(filename, sizeof(filename), "%s/%s.deb.tgz", directory, name);
+
+    if ((tarfile = tar_open(filename, 1)) == NULL)
+      return (1);
+
+   /*
+    * Archive the main package and subpackages...
+    */
+
+    if (tar_package(tarfile, "deb", prodname, directory, platname, dist, NULL))
+    {
+      tar_close(tarfile);
+      return (1);
+    }
+
+    for (i = 0; i < dist->num_subpackages; i ++)
+    {
+      if (tar_package(tarfile, "deb", prodname, directory, platname, dist,
+                      dist->subpackages[i]))
+      {
+	tar_close(tarfile);
+	return (1);
+      }
+    }
+    
+    tar_close(tarfile);
+  }
+
+  return (0);
+}
+
+
+/*
+ * 'make_subpackage()' - Make a subpackage...
+ */
+
+static int				/* O - 0 = success, 1 = fail */
+make_subpackage(const char     *prodname,
+					/* I - Product short name */
+                const char     *directory,
+					/* I - Directory for distribution files */
+                const char     *platname,
+					/* I - Platform name */
+                dist_t         *dist,	/* I - Distribution information */
+	        struct utsname *platform,
+					/* I - Platform information */
+		const char     *subpackage)
+					/* I - Subpackage */
 {
   int			i, j;		/* Looping vars */
   const char		*header;	/* Dependency header string */
   FILE			*fp;		/* Control file */
-  char			name[1024];	/* Full product name */
-  char			filename[1024];	/* Destination filename */
+  char			prodfull[255],	/* Full name of product */
+			name[1024],	/* Full product name */
+			filename[1024];	/* Destination filename */
   command_t		*c;		/* Current command */
   depend_t		*d;		/* Current dependency */
   file_t		*file;		/* Current distribution file */
   struct passwd		*pwd;		/* Pointer to user record */
   struct group		*grp;		/* Pointer to group record */
-  const char		*runlevels;	/* Run levels */
+  const char		*runlevels,	/* Run levels */
+			*rlptr;		/* Pointer into runlevels */
   static const char	*depends[] =	/* Dependency names */
 			{
 			  "Depends:",
@@ -58,21 +160,33 @@ make_deb(const char     *prodname,	/* I - Product short name */
 			};
 
 
-  if (Verbosity)
-    puts("Creating Debian distribution...");
+ /*
+  * Figure out the full name of the distribution...
+  */
 
-  if (dist->relnumber)
-  {
-    if (platname[0])
-      snprintf(name, sizeof(name), "%s-%s-%d-%s", prodname, dist->version, dist->relnumber,
-              platname);
-    else
-      snprintf(name, sizeof(name), "%s-%s-%d", prodname, dist->version, dist->relnumber);
-  }
-  else if (platname[0])
-    snprintf(name, sizeof(name), "%s-%s-%s", prodname, dist->version, platname);
+  if (subpackage)
+    snprintf(prodfull, sizeof(prodfull), "%s-%s", prodname, subpackage);
   else
-    snprintf(name, sizeof(name), "%s-%s", prodname, dist->version);
+    strlcpy(prodfull, prodname, sizeof(prodfull));
+
+ /*
+  * Then the subdirectory name...
+  */
+
+  if (dist->release[0])
+    snprintf(name, sizeof(name), "%s-%s-%s", prodfull, dist->version,
+             dist->release);
+  else
+    snprintf(name, sizeof(name), "%s-%s", prodfull, dist->version);
+
+  if (platname[0])
+  {
+    strlcat(name, "-", sizeof(name));
+    strlcat(name, platname, sizeof(name));
+  }
+
+  if (Verbosity)
+    printf("Creating Debian %s distribution...\n", name);
 
  /*
   * Write the control file for DPKG...
@@ -83,11 +197,11 @@ make_deb(const char     *prodname,	/* I - Product short name */
 
   snprintf(filename, sizeof(filename), "%s/%s", directory, name);
   mkdir(filename, 0777);
-  strcat(filename, "/DEBIAN");
+  strlcat(filename, "/DEBIAN", sizeof(filename));
   mkdir(filename, 0777);
   chmod(filename, 0755);
 
-  strcat(filename, "/control");
+  strlcat(filename, "/control", sizeof(filename));
 
   if ((fp = fopen(filename, "w")) == NULL)
   {
@@ -96,9 +210,9 @@ make_deb(const char     *prodname,	/* I - Product short name */
     return (1);
   }
 
-  fprintf(fp, "Package: %s\n", prodname);
-  if (dist->relnumber)
-    fprintf(fp, "Version: %s-%d\n", dist->version, dist->relnumber);
+  fprintf(fp, "Package: %s\n", prodfull);
+  if (dist->release[0])
+    fprintf(fp, "Version: %s-%s\n", dist->version, dist->release);
   else
     fprintf(fp, "Version: %s\n", dist->version);
   fprintf(fp, "Maintainer: %s\n", dist->vendor);
@@ -108,28 +222,34 @@ make_deb(const char     *prodname,	/* I - Product short name */
   * (which we change in get_platform to a common name)
   */
 
-  if (strcmp(platform->machine, "intel") == 0)
+  if (!strcmp(platform->machine, "intel"))
     fputs("Architecture: i386\n", fp);
+  else if (!strcmp(platform->machine, "ppc"))
+    fputs("Architecture: powerpc\n", fp);
   else
     fprintf(fp, "Architecture: %s\n", platform->machine);
 
   fprintf(fp, "Description: %s\n", dist->product);
   fprintf(fp, " Copyright: %s\n", dist->copyright);
   for (i = 0; i < dist->num_descriptions; i ++)
-    fprintf(fp, " %s\n", dist->descriptions[i]);
+    if (dist->descriptions[i].subpackage == subpackage)
+      fprintf(fp, " %s\n", dist->descriptions[i].description);
 
   for (j = DEPEND_REQUIRES; j <= DEPEND_PROVIDES; j ++)
   {
     for (i = dist->num_depends, d = dist->depends; i > 0; i --, d ++)
-      if (d->type == j)
+      if (d->type == j && d->subpackage == subpackage)
 	break;
 
     if (i)
     {
       for (header = depends[j]; i > 0; i --, d ++, header = ",")
-	if (d->type == j)
+	if (d->type == j && d->subpackage == subpackage)
 	{
-          fprintf(fp, "%s %s", header, d->product);
+	  if (!strcmp(d->product, "_self"))
+            fprintf(fp, "%s %s", header, prodname);
+	  else
+            fprintf(fp, "%s %s", header, d->product);
 
 	  if (d->vernumber[0] == 0)
 	  {
@@ -137,7 +257,12 @@ make_deb(const char     *prodname,	/* I - Product short name */
               fprintf(fp, " (<= %s)", d->version[1]);
 	  }
 	  else
-	    fprintf(fp, " (>= %s, <= %s)", d->version[0], d->version[1]);
+	  {
+	    if (d->vernumber[1] < INT_MAX)
+	      fprintf(fp, " (>= %s, <= %s)", d->version[0], d->version[1]);
+	    else
+	      fprintf(fp, " (>= %s)", d->version[0]);
+	  }
 	}
 
       putc('\n', fp);
@@ -151,7 +276,7 @@ make_deb(const char     *prodname,	/* I - Product short name */
   */
 
   for (i = dist->num_commands, c = dist->commands; i > 0; i --, c ++)
-    if (c->type == COMMAND_PRE_INSTALL)
+    if (c->type == COMMAND_PRE_INSTALL && c->subpackage == subpackage)
       break;
 
   if (i)
@@ -174,7 +299,7 @@ make_deb(const char     *prodname,	/* I - Product short name */
     fputs("# " EPM_VERSION "\n", fp);
 
     for (; i > 0; i --, c ++)
-      if (c->type == COMMAND_PRE_INSTALL)
+      if (c->type == COMMAND_PRE_INSTALL && c->subpackage == subpackage)
         fprintf(fp, "%s\n", c->command);
 
     fclose(fp);
@@ -185,12 +310,12 @@ make_deb(const char     *prodname,	/* I - Product short name */
   */
 
   for (i = dist->num_commands, c = dist->commands; i > 0; i --, c ++)
-    if (c->type == COMMAND_POST_INSTALL)
+    if (c->type == COMMAND_POST_INSTALL && c->subpackage == subpackage)
       break;
 
   if (!i)
     for (i = dist->num_files, file = dist->files; i > 0; i --, file ++)
-      if (tolower(file->type) == 'i')
+      if (tolower(file->type) == 'i' && file->subpackage == subpackage)
         break;
 
   if (i)
@@ -213,24 +338,25 @@ make_deb(const char     *prodname,	/* I - Product short name */
     fputs("# " EPM_VERSION "\n", fp);
 
     for (i = dist->num_commands, c = dist->commands; i > 0; i --, c ++)
-      if (c->type == COMMAND_POST_INSTALL)
+      if (c->type == COMMAND_POST_INSTALL && c->subpackage == subpackage)
         fprintf(fp, "%s\n", c->command);
 
     for (i = dist->num_files, file = dist->files; i > 0; i --, file ++)
-      if (tolower(file->type) == 'i')
+      if (tolower(file->type) == 'i' && file->subpackage == subpackage)
       {
-        for (runlevels = get_runlevels(file, "02345");
-	     isdigit(*runlevels);
-	     runlevels ++)
-	{
-	  if (*runlevels == '0')
-            fprintf(fp, "update-rc.d %s stop %02d 0 >/dev/null\n", file->dst,
-	            get_stop(file, 0));
-          else
-            fprintf(fp, "update-rc.d %s start %02d %c >/dev/null\n", file->dst,
-	            get_start(file, 99), *runlevels);
-        }
+        runlevels = get_runlevels(file, "02345");
 
+        fprintf(fp, "update-rc.d %s start %02d", file->dst,
+	        get_start(file, 99));
+
+        for (rlptr = runlevels; isdigit(*rlptr & 255); rlptr ++)
+	  if (*rlptr != '0')
+	    fprintf(fp, " %c", *rlptr);
+
+	if (strchr(runlevels, '0') != NULL)
+          fprintf(fp, " . stop %02d 0", get_stop(file, 0));
+
+        fputs(" . >/dev/null\n", fp);
         fprintf(fp, "/etc/init.d/%s start\n", file->dst);
       }
 
@@ -242,12 +368,12 @@ make_deb(const char     *prodname,	/* I - Product short name */
   */
 
   for (i = dist->num_commands, c = dist->commands; i > 0; i --, c ++)
-    if (c->type == COMMAND_PRE_REMOVE)
+    if (c->type == COMMAND_PRE_REMOVE && c->subpackage == subpackage)
       break;
 
   if (!i)
     for (i = dist->num_files, file = dist->files; i > 0; i --, file ++)
-      if (tolower(file->type) == 'i')
+      if (tolower(file->type) == 'i' && file->subpackage == subpackage)
         break;
 
   if (i)
@@ -270,19 +396,13 @@ make_deb(const char     *prodname,	/* I - Product short name */
     fputs("# " EPM_VERSION "\n", fp);
 
     for (i = dist->num_commands, c = dist->commands; i > 0; i --, c ++)
-      if (c->type == COMMAND_PRE_REMOVE)
+      if (c->type == COMMAND_PRE_REMOVE && c->subpackage == subpackage)
         fprintf(fp, "%s\n", c->command);
 
     for (i = dist->num_files, file = dist->files; i > 0; i --, file ++)
-      if (tolower(file->type) == 'i')
-      {
+      if (tolower(file->type) == 'i' && file->subpackage == subpackage)
         fprintf(fp, "/etc/init.d/%s stop\n", file->dst);
 
-        fputs("if [ purge = \"$1\" ]; then\n", fp);
-        fprintf(fp, "	update-rc.d %s remove >/dev/null\n", file->dst);
-        fputs("fi\n", fp);
-      }
-
     fclose(fp);
   }
 
@@ -291,9 +411,14 @@ make_deb(const char     *prodname,	/* I - Product short name */
   */
 
   for (i = dist->num_commands, c = dist->commands; i > 0; i --, c ++)
-    if (c->type == COMMAND_POST_REMOVE)
+    if (c->type == COMMAND_POST_REMOVE && c->subpackage == subpackage)
       break;
 
+  if (!i)
+    for (i = dist->num_files, file = dist->files; i > 0; i --, file ++)
+      if (tolower(file->type) == 'i' && file->subpackage == subpackage)
+        break;
+
   if (i)
   {
     if (Verbosity)
@@ -313,9 +438,17 @@ make_deb(const char     *prodname,	/* I - Product short name */
     fputs("#!/bin/sh\n", fp);
     fputs("# " EPM_VERSION "\n", fp);
 
-    for (; i > 0; i --, c ++)
-      if (c->type == COMMAND_POST_REMOVE)
-        fprintf(fp, "%s\n", c->command);
+    for (i = dist->num_commands, c = dist->commands; i > 0; i --, c ++)
+      if (c->type == COMMAND_POST_REMOVE && c->subpackage == subpackage)
+	fprintf(fp, "%s\n", c->command);
+
+    for (i = dist->num_files, file = dist->files; i > 0; i --, file ++)
+      if (tolower(file->type) == 'i' && file->subpackage == subpackage)
+      {
+        fputs("if [ purge = \"$1\" ]; then\n", fp);
+        fprintf(fp, "	update-rc.d %s remove >/dev/null\n", file->dst);
+        fputs("fi\n", fp);
+      }
 
     fclose(fp);
   }
@@ -337,9 +470,9 @@ make_deb(const char     *prodname,	/* I - Product short name */
   }
 
   for (i = dist->num_files, file = dist->files; i > 0; i --, file ++)
-    if (tolower(file->type) == 'c')
+    if (tolower(file->type) == 'c' && file->subpackage == subpackage)
       fprintf(fp, "%s\n", file->dst);
-    else if (tolower(file->type) == 'i')
+    else if (tolower(file->type) == 'i' && file->subpackage == subpackage)
       fprintf(fp, "/etc/init.d/%s\n", file->dst);
 
   fclose(fp);
@@ -353,6 +486,9 @@ make_deb(const char     *prodname,	/* I - Product short name */
 
   for (i = dist->num_files, file = dist->files; i > 0; i --, file ++)
   {
+    if (file->subpackage != subpackage)
+      continue;
+
    /*
     * Find the username and groupname IDs...
     */
@@ -371,7 +507,8 @@ make_deb(const char     *prodname,	/* I - Product short name */
     {
       case 'c' :
       case 'f' :
-          snprintf(filename, sizeof(filename), "%s/%s%s", directory, name, file->dst);
+          snprintf(filename, sizeof(filename), "%s/%s%s", directory, name,
+	           file->dst);
 
 	  if (Verbosity > 1)
 	    printf("%s -> %s...\n", file->src, filename);
@@ -381,7 +518,8 @@ make_deb(const char     *prodname,	/* I - Product short name */
 	    return (1);
           break;
       case 'i' :
-          snprintf(filename, sizeof(filename), "%s/%s/etc/init.d/%s", directory, name, file->dst);
+          snprintf(filename, sizeof(filename), "%s/%s/etc/init.d/%s",
+	           directory, name, file->dst);
 
 	  if (Verbosity > 1)
 	    printf("%s -> %s...\n", file->src, filename);
@@ -391,7 +529,8 @@ make_deb(const char     *prodname,	/* I - Product short name */
 	    return (1);
           break;
       case 'd' :
-          snprintf(filename, sizeof(filename), "%s/%s%s", directory, name, file->dst);
+          snprintf(filename, sizeof(filename), "%s/%s%s", directory, name,
+	           file->dst);
 
 	  if (Verbosity > 1)
 	    printf("Directory %s...\n", filename);
@@ -400,7 +539,8 @@ make_deb(const char     *prodname,	/* I - Product short name */
 			 grp ? grp->gr_gid : 0);
           break;
       case 'l' :
-          snprintf(filename, sizeof(filename), "%s/%s%s", directory, name, file->dst);
+          snprintf(filename, sizeof(filename), "%s/%s%s", directory, name,
+	           file->dst);
 
 	  if (Verbosity > 1)
 	    printf("%s -> %s...\n", file->src, filename);
@@ -415,7 +555,7 @@ make_deb(const char     *prodname,	/* I - Product short name */
   */
 
   if (Verbosity)
-    puts("Building Debian binary distribution...");
+    printf("Building Debian %s binary distribution...\n", name);
 
   if (run_command(directory, "dpkg --build %s", name))
     return (1);
@@ -427,7 +567,7 @@ make_deb(const char     *prodname,	/* I - Product short name */
   if (!KeepFiles)
   {
     if (Verbosity)
-      puts("Removing temporary distribution files...");
+      printf("Removing temporary %s distribution files...\n", name);
 
     run_command(NULL, "/bin/rm -rf %s/%s", directory, name);
   }
@@ -437,5 +577,5 @@ make_deb(const char     *prodname,	/* I - Product short name */
 
 
 /*
- * End of "$Id: deb.c,v 1.19 2002/10/17 14:56:13 mike Exp $".
+ * End of "$Id: deb.c 670 2006-04-05 15:08:08Z mike $".
  */
