@@ -1,9 +1,9 @@
 /*
- * "$Id: epm.c,v 1.72 2002/10/18 14:57:48 mike Exp $"
+ * "$Id: epm.c 677 2006-04-05 21:09:58Z mike $"
  *
  *   Main program source for the ESP Package Manager (EPM).
  *
- *   Copyright 1999-2002 by Easy Software Products.
+ *   Copyright 1999-2006 by Easy Software Products.
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -17,9 +17,10 @@
  *
  * Contents:
  *
- *   main()  - Read a product list and produce a distribution.
- *   info()  - Show the EPM copyright and license.
- *   usage() - Show command-line usage instructions.
+ *   main()   - Read a product list and produce a distribution.
+ *   depend() - Show dependencies.
+ *   info()   - Show the EPM copyright and license.
+ *   usage()  - Show command-line usage instructions.
  */
 
 /*
@@ -33,16 +34,20 @@
  * Globals...
  */
 
-int		Verbosity = 0;
+int		CompressFiles = EPM_COMPRESS;
+const char	*DataDir = EPM_DATADIR;
 int		KeepFiles = 0;
 const char	*SetupProgram = EPM_LIBDIR "/setup";
 const char	*SoftwareDir = EPM_SOFTWARE;
+const char	*UninstProgram = EPM_LIBDIR "/uninst";
+int		Verbosity = 0;
 
 
 /*
  * Local functions...
  */
 
+static void	depend(dist_t *dist);
 static void	info(void);
 static void	usage(void);
 
@@ -51,24 +56,26 @@ static void	usage(void);
  * 'main()' - Read a product list and produce a distribution.
  */
 
-int				/* O - Exit status */
-main(int  argc,			/* I - Number of command-line arguments */
-     char *argv[])		/* I - Command-line arguments */
+int					/* O - Exit status */
+main(int  argc,				/* I - Number of command-line args */
+     char *argv[])			/* I - Command-line arguments */
 {
-  int		i;		/* Looping var */
-  int		strip;		/* 1 if we should strip executables */
-  struct utsname platform;	/* UNIX name info */
-  char		*namefmt,	/* Name format to use */
-		platname[255],	/* Base platform name */
-		prodname[256],	/* Product name */
-		listname[256],	/* List file name */
-		directory[255],	/* Name of install directory */
-		*temp,		/* Temporary string pointer */
-		*setup,		/* Setup GUI image */
-		*types;		/* Setup GUI install types */
-  dist_t	*dist;		/* Software distribution */
-  int		format;		/* Distribution format */
-  static char	*formats[] =	/* Distribution format strings */
+  int		i;			/* Looping var */
+  int		strip;			/* 1 if we should strip executables */
+  struct utsname platform;		/* UNIX name info */
+  char		*namefmt,		/* Name format to use */
+		*custom_name,		/* User-supplied system name */
+		platname[255],		/* Base platform name */
+		prodname[256],		/* Product name */
+		listname[256],		/* List file name */
+		directory[255],		/* Name of install directory */
+		*temp,			/* Temporary string pointer */
+		*setup,			/* Setup GUI image */
+		*types;			/* Setup GUI install types */
+  dist_t	*dist;			/* Software distribution */
+  int		format;			/* Distribution format */
+  int		show_depend;		/* Show dependencies */
+  static char	*formats[] =		/* Distribution format strings */
 		{
 		  "portable",
 		  "aix",
@@ -79,6 +86,7 @@ main(int  argc,			/* I - Number of command-line arguments */
 		  "pkg",
 		  "rpm",
 		  "setld",
+		  "slackware",
 		  "swinstall"
 		};
 
@@ -104,9 +112,11 @@ main(int  argc,			/* I - Number of command-line arguments */
   setup        = NULL;
   types        = NULL;
   namefmt      = "srm";
+  custom_name  = 0;
   prodname[0]  = '\0';
   listname[0]  = '\0';
   directory[0] = '\0';
+  show_depend  = 0;
 
   for (i = 1; i < argc; i ++)
     if (argv[i][0] == '-')
@@ -132,8 +142,7 @@ main(int  argc,			/* I - Number of command-line arguments */
               temp = argv[i];
 	    }
 
-	    strncpy(platform.machine, temp, sizeof(platform.machine) - 1);
-	    platform.machine[sizeof(platform.machine) - 1] = '\0';
+	    strlcpy(platform.machine, temp, sizeof(platform.machine));
 	    break;
 
         case 'f' : /* Format */
@@ -170,6 +179,8 @@ main(int  argc,			/* I - Number of command-line arguments */
 	      format = PACKAGE_RPM;
 	    else if (strcasecmp(temp, "setld") == 0)
 	      format = PACKAGE_SETLD;
+	    else if (strcasecmp(temp, "slackware") == 0)
+	      format = PACKAGE_SLACKWARE;
 	    else if (strcasecmp(temp, "swinstall") == 0 ||
 	             strcasecmp(temp, "depot") == 0)
 	      format = PACKAGE_SWINSTALL;
@@ -217,6 +228,22 @@ main(int  argc,			/* I - Number of command-line arguments */
 	    KeepFiles = 1;
 	    break;
 
+	case 'm' : /* Custom sysname-release-machine string */
+	    if (argv[i][2])
+	      custom_name = argv[i] + 2;
+	    else
+	    {
+	      i ++;
+	      if (i >= argc)
+	      {
+	        puts("epm: Expected custom system name string.");
+	        usage();
+	      }
+
+	      custom_name = argv[i];
+	    }
+	    break;
+
         case 'n' : /* Name with sysname, machine, and/or release */
 	    namefmt = argv[i] + 2;
 	    break;
@@ -242,14 +269,46 @@ main(int  argc,			/* I - Number of command-line arguments */
 	          stderr);
 	    break;
 
+        case 'u' : /* Uncompressed output */
+	    CompressFiles = 0;
+	    break;
+
         case 'v' : /* Be verbose */
 	    Verbosity += strlen(argv[i]) - 1;
 	    break;
 
+        case 'z' : /* Compress output */
+	    CompressFiles = 1;
+	    break;
+
         case '-' : /* --option */
-	    if (strcmp(argv[i], "--keep-files") == 0)
+	    if (!strcmp(argv[i], "--data-dir"))
+	    {
+	      i ++;
+	      if (i < argc)
+	        DataDir = argv[i];
+	      else
+	      {
+		puts("epm: Expected data directory.");
+		usage();
+	      }
+	    }
+	    else if (!strcmp(argv[i], "--depend"))
+	      show_depend = 1;
+	    else if (!strcmp(argv[i], "--keep-files"))
 	      KeepFiles = 1;
-	    else if (strcmp(argv[i], "--setup-image") == 0)
+	    else if (!strcmp(argv[i], "--output-dir"))
+	    {
+	      i ++;
+	      if (i < argc)
+		strlcpy(directory, argv[i], sizeof(directory));
+	      else
+	      {
+		puts("epm: Expected output directory.");
+		usage();
+	      }
+	    }
+	    else if (!strcmp(argv[i], "--setup-image"))
 	    {
 	      i ++;
 	      if (i < argc)
@@ -260,7 +319,7 @@ main(int  argc,			/* I - Number of command-line arguments */
 	        usage();
               }
             }
-	    else if (strcmp(argv[i], "--setup-program") == 0)
+	    else if (!strcmp(argv[i], "--setup-program"))
 	    {
 	      i ++;
 	      if (i < argc)
@@ -271,7 +330,7 @@ main(int  argc,			/* I - Number of command-line arguments */
 	        usage();
               }
             }
-	    else if (strcmp(argv[i], "--setup-types") == 0)
+	    else if (!strcmp(argv[i], "--setup-types"))
 	    {
 	      i ++;
 	      if (i < argc)
@@ -282,7 +341,7 @@ main(int  argc,			/* I - Number of command-line arguments */
 	        usage();
               }
             }
-	    else if (strcmp(argv[i], "--software-dir") == 0)
+	    else if (!strcmp(argv[i], "--software-dir"))
 	    {
 	      i ++;
 	      if (i < argc)
@@ -293,20 +352,22 @@ main(int  argc,			/* I - Number of command-line arguments */
 	        usage();
               }
             }
-            else if (strcmp(argv[i], "--output-dir") == 0)
-            {
-              i ++;
-              if (i < argc)
+	    else if (!strcmp(argv[i], "--uninstall-program"))
+	    {
+	      i ++;
+	      if (i < argc)
+	        UninstProgram = argv[i];
+	      else
 	      {
-                strncpy(directory, argv[i], sizeof(directory) - 1);
-		directory[sizeof(directory) - 1] = '\0';
+		puts("epm: Expected uninstall program.");
+		usage();
 	      }
-              else
-              {
-                puts("epm: Expected output directory.");
-                usage();
-              }
-            }
+	    }
+	    else if (!strcmp(argv[i], "--version"))
+	    {
+	      info();
+	      return (0);
+	    }
 	    else
             {
               printf("epm: Unknown option \"%s\".\n", argv[i]);
@@ -342,6 +403,13 @@ main(int  argc,			/* I - Number of command-line arguments */
     usage();
   }
 
+  for (i = 0; prodname[i]; i ++)
+    if (!isalnum(prodname[i] & 255))
+    {
+      puts("epm: Product names can only contain letters and numbers!");
+      usage();
+    }
+
   if (!listname[0])
     snprintf(listname, sizeof(listname), "%s.list", prodname);
 
@@ -355,28 +423,36 @@ main(int  argc,			/* I - Number of command-line arguments */
     * User did not specify an output directory, so use our default...
     */
 
-    snprintf(directory, sizeof(directory), "%s-%s-%s", platform.sysname,
+    if (custom_name)
+      sprintf(directory, "%s", custom_name);
+    else
+      snprintf(directory, sizeof(directory), "%s-%s-%s", platform.sysname,
              platform.release, platform.machine);
   }
 
   platname[0] = '\0';
 
-  for (temp = namefmt; *temp != '\0'; temp ++)
+  if (custom_name)
+    strlcat(platname, custom_name, sizeof(platname));
+  else
   {
-    if (platname[0])
-      strcat(platname, "-");
-
-    if (*temp == 'm')
-      strcat(platname, platform.machine);
-    else if (*temp == 'r')
-      strcat(platname, platform.release);
-    else if (*temp == 's')
-      strcat(platname, platform.sysname);
-    else
+    for (temp = namefmt; *temp != '\0'; temp ++)
     {
-      printf("epm: Bad name format character \"%c\" in \"%s\".\n", *temp,
-             namefmt);
-      usage();
+      if (platname[0])
+        strlcat(platname, "-", sizeof(platname));
+
+      if (*temp == 'm')
+        strlcat(platname, platform.machine, sizeof(platname));
+      else if (*temp == 'r')
+        strlcat(platname, platform.release, sizeof(platname));
+      else if (*temp == 's')
+        strlcat(platname, platform.sysname, sizeof(platname));
+      else
+      {
+        printf("epm: Bad name format character \"%c\" in \"%s\".\n", *temp,
+               namefmt);
+        usage();
+      }
     }
   }
 
@@ -401,8 +477,7 @@ main(int  argc,			/* I - Number of command-line arguments */
   if (!dist->product[0] ||
       !dist->copyright[0] ||
       !dist->vendor[0] ||
-      !dist->license[0] ||
-      !dist->readme[0] ||
+      (!dist->license[0] && !dist->readme[0]) ||
       !dist->version[0])
   {
     fputs("epm: Error - missing %product, %copyright, %vendor, %license,\n", stderr);
@@ -422,6 +497,19 @@ main(int  argc,			/* I - Number of command-line arguments */
     return (1);
   }
 
+ /*
+  * Show dependencies?
+  */
+
+  if (show_depend)
+  {
+    depend(dist);
+
+    free_dist(dist);
+
+    return (0);
+  }
+
  /*
   * Strip executables as needed...
   */
@@ -456,6 +544,9 @@ main(int  argc,			/* I - Number of command-line arguments */
     case PACKAGE_BSD :
         i = make_bsd(prodname, directory, platname, dist, &platform);
 	break;
+	case PACKAGE_SLACKWARE :
+        i = make_slackware(prodname, directory, platname, dist, &platform);
+	break;
     case PACKAGE_DEB :
         if (geteuid())
 	  fputs("epm: Warning - file permissions and ownership may not be correct\n"
@@ -467,7 +558,7 @@ main(int  argc,			/* I - Number of command-line arguments */
         i = make_inst(prodname, directory, platname, dist, &platform);
 	break;
     case PACKAGE_OSX :
-        i = make_osx(prodname, directory, platname, dist, &platform);
+        i = make_osx(prodname, directory, platname, dist, &platform, setup);
 	break;
     case PACKAGE_PKG :
         i = make_pkg(prodname, directory, platname, dist, &platform);
@@ -508,6 +599,32 @@ main(int  argc,			/* I - Number of command-line arguments */
 }
 
 
+/*
+ * 'depend()' - Show dependencies.
+ */
+
+static void
+depend(dist_t *dist)			/* I - Distribution */
+{
+  int		i;			/* Looping var */
+  file_t	*file;			/* Current file */
+
+
+  for (i = dist->num_files, file = dist->files; i > 0; i --, file ++)
+    switch (file->type)
+    {
+      case 'c' :
+      case 'f' :
+      case 'i' :
+      case 'C' :
+      case 'F' :
+      case 'I' :
+          puts(file->src);
+	  break;
+    }
+}
+
+
 /*
  * 'info()' - Show the EPM copyright and license.
  */
@@ -516,7 +633,7 @@ static void
 info(void)
 {
   puts(EPM_VERSION);
-  puts("Copyright 1999-2002 by Easy Software Products.");
+  puts("Copyright 1999-2006 by Easy Software Products.");
   puts("");
   puts("EPM is free software and comes with ABSOLUTELY NO WARRANTY; for details");
   puts("see the GNU General Public License in the file COPYING or at");
@@ -536,32 +653,59 @@ usage(void)
 
   puts("Usage: epm [options] [name=value ... name=value] product [list-file]");
   puts("Options:");
+  puts("-a arch");
+  puts("    Use the named architecture instead of the local one.");
   puts("-g");
   puts("    Don't strip executables in distributions.");
-  puts("-f {aix,bsd,deb,depot,inst,native,pkg,portable,rpm,setld,swinstall,tardist}");
+  puts("-f {aix,bsd,deb,depot,inst,native,pkg,portable,rpm,setld,slackware,swinstall,tardist}");
   puts("    Set distribution format.");
   puts("-k");
   puts("    Keep intermediate files (spec files, etc.)");
+  puts("-m name");
+  puts("    Set distribution filename to include the specified platform name.");
+  puts("    This overrides \"-n\".");
   puts("-n[mrs]");
   puts("    Set distribution filename to include machine (m), OS release (r),");
   puts("    and/or OS name (s).");
+  puts("-s setup.xpm");
+  puts("    Enable the setup GUI and use \"setup.xpm\" for the setup image.");
+  puts("-u");
+#if EPM_COMPRESS == 0
+  puts("    Do not compress files in packages (default).");
+#else
+  puts("    Do not compress files in packages.");
+#endif /* EPM_COMPRESS == 0 */
   puts("-v");
   puts("    Be verbose.");
-  puts("-s setup.xpm");
+  puts("-z");
+#if EPM_COMPRESS == 1
+  puts("    Compress files in packages (default).");
+#else
+  puts("    Compress files in packages.");
+#endif /* EPM_COMPRESS == 1 */
+  puts("--data-dir /foo/bar/directory");
+  puts("    Use the named setup data file directory instead of " EPM_DATADIR ".");
   puts("--help");
+  puts("    Show this usage message.");
   puts("--keep-files");
+  puts("    Keep temporary distribution files in the output directory.");
   puts("--output-dir /foo/bar/directory");
+  puts("    Enable the setup GUI and use \"setup.xpm\" for the setup image.");
   puts("--setup-image setup.xpm");
   puts("    Enable the setup GUI and use \"setup.xpm\" for the setup image.");
   puts("--setup-program /foo/bar/setup");
   puts("    Use the named setup program instead of " EPM_LIBDIR "/setup.");
   puts("--setup-types setup.types");
   puts("    Include the named setup.types file with the distribution.");
+  puts("--uninstalll-program /foo/bar/uninst");
+  puts("    Use the named uninstall program instead of " EPM_LIBDIR "/uninst.");
+  puts("--version");
+  puts("    Show EPM version.");
 
   exit(1);
 }
 
 
 /*
- * End of "$Id: epm.c,v 1.72 2002/10/18 14:57:48 mike Exp $".
+ * End of "$Id: epm.c 677 2006-04-05 21:09:58Z mike $".
  */
