@@ -1,9 +1,9 @@
 /*
- * "$Id: file.c,v 1.12 2002/10/18 14:57:49 mike Exp $"
+ * "$Id: file.c 611 2005-01-11 21:37:42Z mike $"
  *
  *   File functions for the ESP Package Manager (EPM).
  *
- *   Copyright 1999-2002 by Easy Software Products.
+ *   Copyright 1999-2005 by Easy Software Products.
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -35,18 +35,18 @@
  * 'copy_file()' - Copy a file...
  */
 
-int				/* O - 0 on success, -1 on failure */
-copy_file(const char *dst,	/* I - Destination file */
-          const char *src,	/* I - Source file */
-          int        mode,	/* I - Permissions */
-	  int        owner,	/* I - Owner ID */
-	  int        group)	/* I - Group ID */
+int					/* O - 0 on success, -1 on failure */
+copy_file(const char *dst,		/* I - Destination file */
+          const char *src,		/* I - Source file */
+          mode_t     mode,		/* I - Permissions */
+	  uid_t      owner,		/* I - Owner ID */
+	  gid_t      group)		/* I - Group ID */
 {
-  FILE	*dstfile,	/* Destination file */
-	*srcfile;	/* Source file */
-  char	buffer[8192];	/* Copy buffer */
-  char	*slash;		/* Pointer to trailing slash */
-  int	bytes;		/* Number of bytes read/written */
+  FILE		*dstfile,		/* Destination file */
+		*srcfile;		/* Source file */
+  char		buffer[8192];		/* Copy buffer */
+  char		*slash;			/* Pointer to trailing slash */
+  size_t	bytes;			/* Number of bytes read/written */
 
 
  /*
@@ -90,7 +90,7 @@ copy_file(const char *dst,	/* I - Destination file */
   while ((bytes = fread(buffer, 1, sizeof(buffer), srcfile)) > 0)
     if (fwrite(buffer, 1, bytes, dstfile) < bytes)
     {
-      fprintf(stderr, "epm: Unable to write to \"%s\" -\n     %s\n", src,
+      fprintf(stderr, "epm: Unable to write to \"%s\" -\n     %s\n", dst,
               strerror(errno));
 
       fclose(srcfile);
@@ -120,9 +120,9 @@ copy_file(const char *dst,	/* I - Destination file */
 
 int					/* O - 0 = success, -1 = error */
 make_directory(const char *directory,	/* I - Directory */
-               int        mode,		/* I - Permissions */
-	       int        owner,	/* I - Owner ID */
-	       int        group)	/* I - Group ID */
+               mode_t     mode,		/* I - Permissions */
+	       uid_t      owner,	/* I - Owner ID */
+	       gid_t      group)	/* I - Group ID */
 {
   char	buffer[8192],			/* Filename buffer */
 	*bufptr;			/* Pointer into buffer */
@@ -162,12 +162,12 @@ make_directory(const char *directory,	/* I - Directory */
  * 'make_link()' - Make a symbolic link.
  */
 
-int				/* O - 0 = success, -1 = error */
-make_link(const char *dst,	/* I - Destination file */
-          const char *src)	/* I - Link */
+int					/* O - 0 = success, -1 = error */
+make_link(const char *dst,		/* I - Destination file */
+          const char *src)		/* I - Link */
 {
-  char	buffer[8192],		/* Copy buffer */
-	*slash;			/* Pointer to trailing slash */
+  char	buffer[8192],			/* Copy buffer */
+	*slash;				/* Pointer to trailing slash */
 
 
  /*
@@ -194,12 +194,12 @@ make_link(const char *dst,	/* I - Destination file */
  */
 
 void
-strip_execs(dist_t *dist)	/* I - Distribution to strip... */
+strip_execs(dist_t *dist)		/* I - Distribution to strip... */
 {
-  int		i;		/* Looping var */
-  file_t	*file;		/* Software file */
-  FILE		*fp;		/* File pointer */
-  char		header[4];	/* File header... */
+  int		i;			/* Looping var */
+  file_t	*file;			/* Software file */
+  FILE		*fp;			/* File pointer */
+  char		header[4];		/* File header... */
 
 
  /*
@@ -222,16 +222,19 @@ strip_execs(dist_t *dist)	/* I - Distribution to strip... */
         * Read the first 3 bytes of the file...
 	*/
 
-        fread(header, 1, sizeof(header) - 1, fp);
+        if (fread(header, 1, sizeof(header) - 1, fp) == 0)
+	  continue;
+
 	header[sizeof(header) - 1] = '\0';
 
 	fclose(fp);
 
        /*
-        * Check for "#!/" at the beginning of the file...
+        * Check for "#!/" or "#" + whitespace at the beginning of the file...
 	*/
 
-        if (strncmp(header, "#!/", 3) == 0)
+        if (!strncmp(header, "#!/", 3) ||
+	    (header[0] == '#' && isspace(header[1] & 255)))
 	  continue;
       }
       else
@@ -257,5 +260,5 @@ strip_execs(dist_t *dist)	/* I - Distribution to strip... */
 
 
 /*
- * End of "$Id: file.c,v 1.12 2002/10/18 14:57:49 mike Exp $".
+ * End of "$Id: file.c 611 2005-01-11 21:37:42Z mike $".
  */
