@@ -1,9 +1,9 @@
 /*
- * "$Id: pkg.c,v 1.23 2002/10/17 18:13:14 mike Exp $"
+ * "$Id: pkg.c 670 2006-04-05 15:08:08Z mike $"
  *
  *   AT&T package gateway for the ESP Package Manager (EPM).
  *
- *   Copyright 1999-2002 by Easy Software Products.
+ *   Copyright 1999-2006 by Easy Software Products.
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
  * Contents:
  *
  *   make_pkg() - Make an AT&T software distribution package.
+ *   pkg_path() - Return an absolute path for the prototype file.
  */
 
 /*
@@ -27,6 +28,13 @@
 #include "epm.h"
 
 
+/*
+ * Local functions...
+ */
+
+static const char	*pkg_path(const char *filename, const char *dirname);
+
+
 /*
  * 'make_pkg()' - Make an AT&T software distribution package.
  */
@@ -38,7 +46,7 @@ make_pkg(const char     *prodname,	/* I - Product short name */
          dist_t         *dist,		/* I - Distribution information */
 	 struct utsname *platform)	/* I - Platform information */
 {
-  int		i;			/* Looping vars */
+  int		i;			/* Looping var */
   FILE		*fp;			/* Control file */
   char		name[1024];		/* Full product name */
   char		filename[1024],		/* Destination filename */
@@ -59,13 +67,13 @@ make_pkg(const char     *prodname,	/* I - Product short name */
   if (Verbosity)
     puts("Creating PKG distribution...");
 
-  if (dist->relnumber)
+  if (dist->release[0])
   {
     if (platname[0])
-      snprintf(name, sizeof(name), "%s-%s-%d-%s", prodname, dist->version, dist->relnumber,
+      snprintf(name, sizeof(name), "%s-%s-%s-%s", prodname, dist->version, dist->release,
               platname);
     else
-      snprintf(name, sizeof(name), "%s-%s-%d", prodname, dist->version, dist->relnumber);
+      snprintf(name, sizeof(name), "%s-%s-%s", prodname, dist->version, dist->release);
   }
   else if (platname[0])
     snprintf(name, sizeof(name), "%s-%s-%s", prodname, dist->version, platname);
@@ -102,10 +110,13 @@ make_pkg(const char     *prodname,	/* I - Product short name */
 	  curdate->tm_hour, curdate->tm_min, curdate->tm_sec);
 
   if (dist->num_descriptions > 0)
-    fprintf(fp, "DESC=%s\n", dist->descriptions[0]);
+    fprintf(fp, "DESC=%s\n", dist->descriptions[0].description);
 
   fputs("CATEGORY=application\n", fp);
-  fputs("CLASSES=none\n", fp);
+  fputs("CLASSES=none", fp);
+  for (i = 0; i < dist->num_subpackages; i ++)
+    fprintf(fp, " %s", dist->subpackages[i]);
+  putc('\n', fp);
 
   if (strcmp(platform->machine, "intel") == 0)
     fputs("ARCH=i86pc\n", fp);
@@ -131,7 +142,9 @@ make_pkg(const char     *prodname,	/* I - Product short name */
   }
 
   for (i = dist->num_depends, d = dist->depends; i > 0; i --, d ++)
-    if (d->type == DEPEND_REQUIRES)
+    if (!strcmp(d->product, "_self"))
+      continue;
+    else if (d->type == DEPEND_REQUIRES)
       fprintf(fp, "P %s\n", d->product);
     else
       fprintf(fp, "I %s\n", d->product);
@@ -330,11 +343,11 @@ make_pkg(const char     *prodname,	/* I - Product short name */
       * Make symlinks for all of the selected run levels...
       */
 
-      for (runlevels = get_runlevels(dist->files + i, "03");
-           isdigit(*runlevels);
+      for (runlevels = get_runlevels(dist->files + i, "023");
+           isdigit(*runlevels & 255);
 	   runlevels ++)
       {
-	file = add_file(dist);
+	file = add_file(dist, dist->files[i].subpackage);
 	file->type = 'l';
 	file->mode = 0;
 	strcpy(file->user, "root");
@@ -377,53 +390,49 @@ make_pkg(const char     *prodname,	/* I - Product short name */
     return (1);
   }
 
-  if (dist->license[0] == '/')
-    fprintf(fp, "i copyright=%s\n", dist->license);
-  else
-    fprintf(fp, "i copyright=%s/%s\n", current, dist->license);
+#if 0 /* apparently does not work on Solaris 7... */
+  fprintf(fp, "!search %s\n", current);
+#endif /* 0 */
 
-  fprintf(fp, "i depend=%s/%s/%s.depend\n", current, directory, prodname);
-  fprintf(fp, "i pkginfo=%s/%s/%s.pkginfo\n", current, directory, prodname);
-
-  for (i = dist->num_files, file = dist->files; i > 0; i --, file ++)
-    if (tolower(file->type) == 'i')
-      break;
+  fprintf(fp, "i copyright=%s\n", pkg_path(dist->license, current));
+  fprintf(fp, "i depend=%s/%s.depend\n", pkg_path(directory, current), prodname);
+  fprintf(fp, "i pkginfo=%s/%s.pkginfo\n", pkg_path(directory, current),
+          prodname);
 
   if (preinstall[0])
-    fprintf(fp, "i preinstall=%s/%s\n", current, preinstall);
+    fprintf(fp, "i preinstall=%s\n", pkg_path(preinstall, current));
   if (postinstall[0])
-    fprintf(fp, "i postinstall=%s/%s\n", current, postinstall);
+    fprintf(fp, "i postinstall=%s\n", pkg_path(postinstall, current));
   if (preremove[0])
-    fprintf(fp, "i preremove=%s/%s\n", current, preremove);
+    fprintf(fp, "i preremove=%s\n", pkg_path(preremove, current));
   if (postremove[0])
-    fprintf(fp, "i postremove=%s/%s\n", current, postremove);
+    fprintf(fp, "i postremove=%s\n", pkg_path(postremove, current));
 
   for (i = dist->num_files, file = dist->files; i > 0; i --, file ++)
     switch (tolower(file->type))
     {
       case 'c' :
-          if (file->src[0] == '/')
-            qprintf(fp, "e none %s=%s %04o %s %s\n", file->dst,
-	            file->src, file->mode, file->user, file->group);
-          else
-            qprintf(fp, "e none %s=%s/%s %04o %s %s\n", file->dst,
-	            current, file->src, file->mode, file->user, file->group);
+          qprintf(fp, "e %s %s=%s %04o %s %s\n",
+	          file->subpackage ? file->subpackage : "none",
+	          file->dst, pkg_path(file->src, current),
+		  file->mode, file->user, file->group);
           break;
       case 'd' :
-	  qprintf(fp, "d none %s %04o %s %s\n", file->dst, file->mode,
-		  file->user, file->group);
+	  qprintf(fp, "d %s %s %04o %s %s\n",
+	          file->subpackage ? file->subpackage : "none",
+	          file->dst, file->mode, file->user, file->group);
           break;
       case 'f' :
       case 'i' :
-          if (file->src[0] == '/')
-            qprintf(fp, "f none %s=%s %04o %s %s\n", file->dst,
-	            file->src, file->mode, file->user, file->group);
-          else
-            qprintf(fp, "f none %s=%s/%s %04o %s %s\n", file->dst,
-	            current, file->src, file->mode, file->user, file->group);
+          qprintf(fp, "f %s %s=%s %04o %s %s\n",
+	          file->subpackage ? file->subpackage : "none",
+	          file->dst, pkg_path(file->src, current),
+		  file->mode, file->user, file->group);
           break;
       case 'l' :
-          qprintf(fp, "s none %s=%s\n", file->dst, file->src);
+          qprintf(fp, "s %s %s=%s\n",
+	          file->subpackage ? file->subpackage : "none",
+	          file->dst, file->src);
           break;
     }
 
@@ -445,9 +454,9 @@ make_pkg(const char     *prodname,	/* I - Product short name */
   */
 
   if (Verbosity)
-    puts("Creating tar.gz file for distribution...");
+    puts("Creating .pkg.tgz file for distribution...");
 
-  snprintf(filename, sizeof(filename), "%s/%s.tar.gz", directory, name);
+  snprintf(filename, sizeof(filename), "%s/%s.pkg.tgz", directory, name);
 
   if ((tarfile = tar_open(filename, 1)) == NULL)
     return (1);
@@ -473,6 +482,13 @@ make_pkg(const char     *prodname,	/* I - Product short name */
                   current, directory, name, prodname))
     return (1);
 
+ /*
+  * Compress the package stream file...
+  */
+
+  if (run_command(directory, EPM_GZIP " -v9 %s.pkg", name))
+    return (1);
+
  /*
   * Remove temporary files...
   */
@@ -503,5 +519,24 @@ make_pkg(const char     *prodname,	/* I - Product short name */
 
 
 /*
- * End of "$Id: pkg.c,v 1.23 2002/10/17 18:13:14 mike Exp $".
+ * 'pkg_path()' - Return an absolute path for the prototype file.
+ */
+
+static const char *			/* O - Absolute filename */
+pkg_path(const char *filename,		/* I - Source filename */
+         const char *dirname)		/* I - Source directory */
+{
+  static char	absname[1024];		/* Absolute filename */
+
+
+  if (filename[0] == '/')
+    return (filename);
+
+  snprintf(absname, sizeof(absname), "%s/%s", dirname, filename);
+  return (absname);
+}
+
+
+/*
+ * End of "$Id: pkg.c 670 2006-04-05 15:08:08Z mike $".
  */
