@@ -1,9 +1,9 @@
 //
-// "$Id: setup2.cxx,v 1.30 2002/06/04 19:17:31 mike Exp $"
+// "$Id: setup2.cxx 645 2005-10-14 18:01:51Z mike $"
 //
-//   ESP Software Wizard main entry for the ESP Package Manager (EPM).
+//   ESP Software Installation Wizard main entry for the ESP Package Manager (EPM).
 //
-//   Copyright 1999-2002 by Easy Software Products.
+//   Copyright 1999-2005 by Easy Software Products.
 //
 //   This program is free software; you can redistribute it and/or modify
 //   it under the terms of the GNU General Public License as published by
@@ -21,20 +21,21 @@
 //   get_dists()    - Get a list of available software products.
 //   install_dist() - Install a distribution...
 //   list_cb()      - Handle selections in the software list.
-//   load_image()   - Load the setup image file (setup.xpm)...
-//   load_types()   - Load the setup image file (setup.xpm)...
-//   next_cb()      - Show software selections or install software.
-//   sort_dists()   - Compare two distribution names...
-//   add_string()   - Add a command to an array of commands...
+//   load_image()   - Load the setup image file (setup.gif/xpm)...
+//   load_types()   - Load the installation types from the setup.types file.
 //   log_cb()       - Add one or more lines of text to the installation log.
+//   next_cb()      - Show software selections or install software.
+//   type_cb()      - Handle selections in the type list.
+//   update_size()  - Update the total +/- sizes of the installations.
 //
 
-#define _SETUP2_CXX_
+#define _DEFINE_GLOBALS_
 #include "setup.h"
+#include <FL/Fl_GIF_Image.H>
+#include <FL/Fl_XPM_Image.H>
 #include <FL/x.H>
 #include <FL/filename.H>
 #include <FL/fl_ask.H>
-#include <FL/Fl_XPM_Image.H>
 #include <errno.h>
 #include <ctype.h>
 #include <unistd.h>
@@ -66,6 +67,13 @@ extern int statfs(const char *, struct statfs *);
 }
 #endif // __osf__
 
+#ifdef __APPLE__
+#  include <Security/Authorization.h>
+#  include <Security/AuthorizationTags.h>
+
+AuthorizationRef SetupAuthorizationRef;
+#endif // __APPLE__
+
 
 //
 // Define a C API function type for comparisons...
@@ -80,145 +88,138 @@ typedef int (*compare_func_t)(const void *, const void *);
 // Local functions...
 //
 
-static void	log_cb(int fd, int *fdptr);
-static void	update_sizes(void);
+void	get_dists(const char *d);
+int	install_dist(const dist_t *dist);
+void	load_image(void);
+void	load_types(void);
+void	log_cb(int fd, int *fdptr);
+void	update_sizes(void);
 
 
 //
 // 'main()' - Main entry for software wizard...
 //
 
-int			// O - Exit status
-main(int  argc,		// I - Number of command-line arguments
-     char *argv[])	// I - Command-line arguments
+int				// O - Exit status
+main(int  argc,			// I - Number of command-line arguments
+     char *argv[])		// I - Command-line arguments
 {
-  Fl_Window	*w;	// Main window...
+  Fl_Window	*w;		// Main window...
+  const char	*distdir;	// Distribution directory
 
 
+#if !defined(__hpux) && !defined(__sun) && !defined(__osf) && !defined(_AIX)
+  // Use modern "skin" for modern OS's...
+  Fl::scheme("plastic");
+#endif // !__hpux && !__sun && !__osf && !_AIX
+
+#ifdef __APPLE__
+  // OSX passes an extra command-line option when run from the Finder.
+  // If the first command-line argument is "-psn..." then skip it and use the full path
+  // to the executable to figure out the distribution directory...
+  if (argc > 1)
+  {
+    if (strncmp(argv[1], "-psn", 4) == 0)
+    {
+      char		*ptr;		// Pointer into basedir
+      static char	basedir[1024];	// Base directory (static so it can be used below)
+
+
+      strlcpy(basedir, argv[0], sizeof(basedir));
+      if ((ptr = strrchr(basedir, '/')) != NULL)
+        *ptr = '\0';
+      if ((ptr = strrchr(basedir, '/')) != NULL && !strcasecmp(ptr, "/MacOS"))
+      {
+        // Got the base directory, now add "Resources" to it...
+	*ptr = '\0';
+	strlcat(basedir, "/Resources", sizeof(basedir));
+	distdir = basedir;
+	chdir(basedir);
+      }
+      else
+        distdir = ".";
+
+    }
+    else
+      distdir = argv[1];
+  }
+  else
+    distdir = ".";
+#else
+  // Use the default scheme on this system...
   Fl::scheme(NULL);
 
-  if (getuid() != 0)
-  {
-    fl_alert("You must be logged in as root to run setup!");
-    return (1);
-  }
+  // Get the directory that has the software in it...
+  if (argc > 1)
+    distdir = argv[1];
+  else
+    distdir = ".";
+#endif // __APPLE__
 
   w = make_window();
 
   WelcomePane->show();
   PrevButton->deactivate();
+  NextButton->deactivate();
 
-  get_installed();
-
-  if (argc > 1)
-    get_dists(argv[1]);
-  else
-    get_dists(".");
-
-  load_types();
   load_image();
 
   w->show(1, argv);
 
+  while (!w->visible())
+    Fl::wait();
+
+#ifdef __APPLE__
+  OSStatus status;
+
+  status = AuthorizationCreate(NULL, kAuthorizationEmptyEnvironment,
+                               kAuthorizationFlagDefaults,
+			       &SetupAuthorizationRef);
+  if (status != errAuthorizationSuccess)
+  {
+    fl_alert("You must have administrative priviledges to install this software!");
+    return (1);
+  }
+
+  AuthorizationItem	items = { kAuthorizationRightExecute, 0, NULL, 0 };
+  AuthorizationRights	rights = { 1, &items };
+
+  status = AuthorizationCopyRights(SetupAuthorizationRef, &rights, NULL,
+				   kAuthorizationFlagDefaults |
+				       kAuthorizationFlagInteractionAllowed |
+				       kAuthorizationFlagPreAuthorize |
+				       kAuthorizationFlagExtendRights, NULL);
+
+  if (status != errAuthorizationSuccess)
+  {
+    fl_alert("You must have administrative priviledges to install this software!");
+    return (1);
+  }
+#else
+  if (getuid() != 0)
+  {
+    fl_alert("You must be logged in as root to run setup!");
+    return (1);
+  }
+#endif // __APPLE__
+
+  get_installed();
+  get_dists(distdir);
+
+  load_types();
+
+  NextButton->activate();
+
   Fl::run();
 
+#ifdef __APPLE__
+  AuthorizationFree(SetupAuthorizationRef, kAuthorizationFlagDefaults);
+#endif // __APPLE__
+
   return (0);
 }
 
 
-//
-// 'add_depend()' - Add a dependency to a distribution.
-//
-
-void
-add_depend(dist_t     *d,	// I - Distribution
-           int        type,	// I - Dependency type
-	   const char *name,	// I - Name of product
-	   int        lowver,	// I - Lower version number
-	   int        hiver)	// I - Highre version number
-{
-  depend_t	*temp;		// Pointer to dependency
-
-
-  if (d->num_depends == 0)
-    temp = (depend_t *)malloc(sizeof(depend_t));
-  else
-    temp = (depend_t *)realloc(d->depends,
-                               sizeof(depend_t) * (d->num_depends + 1));
-
-  if (temp == NULL)
-  {
-    perror("setup: Unable to allocate memory for dependency");
-    exit(1);
-  }
-
-  d->depends     = temp;
-  temp           += d->num_depends;
-  d->num_depends ++;
-
-  memset(temp, 0, sizeof(depend_t));
-  strncpy(temp->product, name, sizeof(temp->product) - 1);
-  temp->type         = type;
-  temp->vernumber[0] = lowver;
-  temp->vernumber[1] = hiver;
-}
-
-
-//
-// 'add_dist()' - Add a distribution.
-//
-
-dist_t *			// O - New distribution
-add_dist(int    *num_d,		// IO - Number of distributions
-         dist_t **d)		// IO - Distributions
-{
-  dist_t	*temp;		// Pointer to current distribution
-
-
-  // Add a new distribution entry...
-  if (*num_d == 0)
-    temp = (dist_t *)malloc(sizeof(dist_t));
-  else
-    temp = (dist_t *)realloc(*d, sizeof(dist_t) * (*num_d + 1));
-
-  if (temp == NULL)
-  {
-    perror("setup: Unable to allocate memory for distribution");
-    exit(1);
-  }
-
-  *d = temp;
-  temp  += *num_d;
-  (*num_d) ++;
-
-  memset(temp, 0, sizeof(dist_t));
-
-  return (temp);
-}
-
-
-//
-// 'find_dist()' - Find a distribution.
-//
-
-dist_t *			// O - Pointer to distribution or NULL
-find_dist(const char *name,	// I - Distribution name
-          int        num_d,	// I - Number of distributions
-	  dist_t     *d)	// I - Distributions
-{
-  while (num_d > 0)
-  {
-    if (strcmp(name, d->product) == 0)
-      return (d);
-
-    d ++;
-    num_d --;
-  }
-
-  return (NULL);
-}
-
-
 //
 // 'get_dists()' - Get a list of available software products.
 //
@@ -258,7 +259,7 @@ get_dists(const char *d)	// I - Directory to look in
   for (i = 0; i < num_files; i ++)
   {
     ext = fl_filename_ext(files[i]->d_name);
-    if (strcmp(ext, ".install") == 0)
+    if (!strcmp(ext, ".install") || !strcmp(ext, ".patch"))
     {
       // Found a .install script...
       if ((fp = fopen(files[i]->d_name, "r")) == NULL)
@@ -314,7 +315,7 @@ get_dists(const char *d)	// I - Directory to look in
 
   if (NumDists == 0)
   {
-    fputs("setup: No distributions to install!\n", stderr);
+    fl_alert("No software found to install!");
     exit(1);
   }
 
@@ -351,94 +352,23 @@ get_dists(const char *d)	// I - Directory to look in
 }
 
 
-//
-// 'get_installed()' - Get a list of installed software products.
-//
-
-void
-get_installed(void)
-{
-  int		i;		// Looping var
-  int		num_files;	// Number of files
-  dirent	**files;	// Files
-  const char	*ext;		// Extension
-  dist_t	*temp;		// Pointer to current distribution
-  FILE		*fp;		// File to read from
-  char		line[1024];	// Line from file...
-
-
-  // See if there are any installed files...
-  NumInstalled = 0;
-  Installed    = NULL;
-
-  if ((num_files = fl_filename_list(EPM_SOFTWARE, &files)) == 0)
-    return;
-
-  // Build a distribution list...
-  for (i = 0; i < num_files; i ++)
-  {
-    ext = fl_filename_ext(files[i]->d_name);
-    if (strcmp(ext, ".remove") == 0)
-    {
-      // Found a .remove script...
-      snprintf(line, sizeof(line), EPM_SOFTWARE "/%s", files[i]->d_name);
-      if ((fp = fopen(line, "r")) == NULL)
-      {
-        perror("setup: Unable to open removal script");
-	exit(1);
-      }
-
-      // Add a new distribution entry...
-      temp = add_dist(&NumInstalled, &Installed);
-
-      strncpy(temp->product, files[i]->d_name, sizeof(temp->product) - 1);
-      *strrchr(temp->product, '.') = '\0';	// Drop .remove
-
-      // Read info from the removal script...
-      while (fgets(line, sizeof(line), fp) != NULL)
-      {
-        // Only read distribution info lines...
-        if (strncmp(line, "#%", 2))
-	  continue;
-
-        // Drop the trailing newline...
-        line[strlen(line) - 1] = '\0';
-
-        // Copy data as needed...
-	if (strncmp(line, "#%product ", 10) == 0)
-	  strncpy(temp->name, line + 10, sizeof(temp->name) - 1);
-	else if (strncmp(line, "#%version ", 10) == 0)
-	  sscanf(line + 10, "%31s%d", temp->version, &(temp->vernumber));
-	else if (strncmp(line, "#%rootsize ", 11) == 0)
-	  temp->rootsize = atoi(line + 11);
-	else if (strncmp(line, "#%usrsize ", 10) == 0)
-	  temp->usrsize = atoi(line + 10);
-      }
-
-      fclose(fp);
-    }
-
-    free(files[i]);
-  }
-
-  free(files);
-}
-
-
 //
 // 'install_dist()' - Install a distribution...
 //
 
-int				// O - Install status
-install_dist(const dist_t *dist)// I - Distribution to install
+int					// O - Install status
+install_dist(const dist_t *dist)	// I - Distribution to install
 {
-  char		command[1024];	// Command string
-  int		fds[2];		// Pipe FDs
-  int		status;		// Exit status
-  int		pid;		// Process ID
-  char		licfile[1024];	// License filename
-  struct stat	licinfo;	// License file info
-  static int	liclength = 0;	// Size of license file
+  char		command[1024];		// Command string
+  int		fds[2];			// Pipe FDs
+  int		status;			// Exit status
+#ifndef __APPLE__
+  int		pid;			// Process ID
+#endif // !__APPLE__
+  char		licfile[1024];		// License filename
+  struct stat	licinfo;		// License file info
+  static int	liclength = 0;		// Size of license file
+  static char	liclabel[1024];		// Label for license pane
 
 
   sprintf(command, "**** %s ****", dist->name);
@@ -452,8 +382,8 @@ install_dist(const dist_t *dist)// I - Distribution to install
     liclength = licinfo.st_size;
 
     // Set the title string...
-    snprintf(command, sizeof(command), "Software License for %s", dist->name);
-    LicenseLabel->label(command);
+    snprintf(liclabel, sizeof(liclabel), "Software License for %s", dist->name);
+    LicensePane->label(liclabel);
 
     // Load the license into the browser...
     LicenseBrowser->clear();
@@ -483,9 +413,31 @@ install_dist(const dist_t *dist)// I - Distribution to install
     }
   }
 
+  sprintf(command, "%s.install", dist->product);
+  if (access(command, 0))
+    sprintf(command, "%s.patch", dist->product);
+
+#ifdef __APPLE__
+  // Run the install script using Apple's authorization API...
+  FILE		*fp = NULL;
+  char		*args[2] = { "now", NULL };
+  OSStatus	astatus;
+
+
+  astatus = AuthorizationExecuteWithPrivileges(SetupAuthorizationRef, command,
+                                               kAuthorizationFlagDefaults,
+                                               args, &fp);
+
+  if (astatus != errAuthorizationSuccess)
+  {
+    InstallLog->add("Failed to execute install script!");
+    return (1);
+  }
+
+  fds[0] = fileno(fp);
+#else
   // Fork the command and redirect errors and info to stdout...
   pipe(fds);
-  sprintf(command, "%s.install", dist->product);
 
   if ((pid = fork()) == 0)
   {
@@ -520,21 +472,31 @@ install_dist(const dist_t *dist)// I - Distribution to install
 
   // Close the output pipe (used by the child)...
   close(fds[1]);
+#endif // __APPLE__
 
   // Listen for data on the input pipe...
   Fl::add_fd(fds[0], (void (*)(int, void *))log_cb, fds);
 
+  // Show the user that we're busy...
+  SetupWindow->cursor(FL_CURSOR_WAIT);
+
   // Loop until the child is done...
   while (fds[0])	// log_cb() will close and zero fds[0]...
   {
     // Wait for events...
     Fl::wait();
 
+#ifndef __APPLE__
     // Check to see if the child went away...
     if (waitpid(0, &status, WNOHANG) == pid)
       break;
+#endif // !__APPLE__
   }
 
+#ifdef __APPLE__
+  fclose(fp);
+#endif // __APPLE__
+
   if (fds[0])
   {
     // Close the pipe - have all the data from the child...
@@ -547,6 +509,9 @@ install_dist(const dist_t *dist)// I - Distribution to install
     wait(&status);
   }
 
+  // Show the user that we're ready...
+  SetupWindow->cursor(FL_CURSOR_DEFAULT);
+
   // Return...
   return (status);
 }
@@ -649,18 +614,24 @@ list_cb(Fl_Check_Browser *, void *)
 
 
 //
-// 'load_image()' - Load the setup image file (setup.xpm)...
+// 'load_image()' - Load the setup image file (setup.gif/xpm)...
 //
 
 void
 load_image(void)
 {
-  Fl_XPM_Image	*xpm;		// New image
+  Fl_Image	*img;			// New image
 
 
-  xpm = new Fl_XPM_Image("setup.xpm");
+  if (!access("setup.xpm", 0))
+    img = new Fl_XPM_Image("setup.xpm");
+  else if (!access("setup.gif", 0))
+    img = new Fl_GIF_Image("setup.gif");
+  else
+    img = NULL;
 
-  WelcomeImage->image(xpm);
+  if (img)
+    WelcomeImage->image(img);
 }
 
 
@@ -674,7 +645,8 @@ load_types(void)
   int		i;		// Looping var
   FILE		*fp;		// File to read from
   char		line[1024],	// Line from file
-		*lineptr;	// Pointer into line
+		*lineptr,	// Pointer into line
+		*sep;		// Separator
   dtype_t	*dt;		// Current install type
   dist_t	*dist;		// Distribution
 
@@ -694,7 +666,7 @@ load_types(void)
       if (*lineptr == '\n')
         *lineptr = '\0';
 
-      if (strncasecmp(line, "TYPE", 4) == 0 && isspace(line[4]))
+      if (!strncasecmp(line, "TYPE", 4) && isspace(line[4]))
       {
         // New type...
 	if (NumInstTypes >= (int)(sizeof(InstTypes) / sizeof(InstTypes[0])))
@@ -705,15 +677,23 @@ load_types(void)
 	  exit(1);
 	}
 
+        // Skip whitespace...
         for (lineptr = line + 5; isspace(*lineptr); lineptr ++);
 
+	// Copy name and label string...
         dt = InstTypes + NumInstTypes;
 	NumInstTypes ++;
 
 	memset(dt, 0, sizeof(dtype_t));
-	strncpy(dt->label, lineptr, sizeof(dt->label) - 10);
+	if ((sep = strchr(lineptr, '/')) != NULL)
+	  *sep++ = '\0';
+	else
+	  sep = lineptr;
+
+	strncpy(dt->name, lineptr, sizeof(dt->name));
+	strncpy(dt->label, sep, sizeof(dt->label) - 10);
       }
-      else if (strncasecmp(line, "INSTALL", 7) == 0 && dt && isspace(line[7]))
+      else if (!strncasecmp(line, "INSTALL", 7) && dt && isspace(line[7]))
       {
         // Install a product...
 	if (dt->num_products >= (int)(sizeof(dt->products) / sizeof(dt->products[0])))
@@ -724,8 +704,10 @@ load_types(void)
 	  exit(1);
 	}
 
+        // Skip whitespace...
         for (lineptr = line + 8; isspace(*lineptr); lineptr ++);
 
+	// Add product to list...
         if ((dist = find_dist(lineptr, NumDists, Dists)) != NULL)
 	{
           dt->products[dt->num_products] = dist - Dists;
@@ -758,7 +740,11 @@ load_types(void)
     else if (dt->size)
       sprintf(dt->label + strlen(dt->label), " (+%dk disk space)", dt->size);
 
-    TypeButton[i]->label(dt->label);
+    if ((lineptr = strchr(dt->label, '/')) != NULL)
+      TypeButton[i]->label(lineptr + 1);
+    else
+      TypeButton[i]->label(dt->label);
+
     TypeButton[i]->show();
   }
 
@@ -767,6 +753,57 @@ load_types(void)
 }
 
 
+//
+// 'log_cb()' - Add one or more lines of text to the installation log.
+//
+
+void
+log_cb(int fd,			// I - Pipe to read from
+       int *fdptr)		// O - Pipe to read from
+{
+  int		bytes;		// Bytes read/to read
+  char		*bufptr;	// Pointer into buffer
+  static int	bufused = 0;	// Number of bytes used
+  static char	buffer[8193];	// Buffer
+
+
+  bytes = 8192 - bufused;
+  if ((bytes = read(fd, buffer + bufused, bytes)) <= 0)
+  {
+    // End of file; zero the FD to tell the install_dist() function to
+    // stop...
+
+    Fl::remove_fd(fd);
+    close(fd);
+    *fdptr = 0;
+
+    if (bufused > 0)
+    {
+      // Add remaining text...
+      buffer[bufused] = '\0';
+      InstallLog->add(buffer);
+      bufused = 0;
+    }
+  }
+  else
+  {
+    // Add bytes to the buffer, then add lines as needed...
+    bufused += bytes;
+    buffer[bufused] = '\0';
+
+    while ((bufptr = strchr(buffer, '\n')) != NULL)
+    {
+      *bufptr++ = '\0';
+      InstallLog->add(buffer);
+      strcpy(buffer, bufptr);
+      bufused -= bufptr - buffer;
+    }
+  }
+
+  InstallLog->bottomline(InstallLog->size());
+}
+
+
 //
 // 'next_cb()' - Show software selections or install software.
 //
@@ -774,11 +811,12 @@ load_types(void)
 void
 next_cb(Fl_Button *, void *)
 {
-  int		i;		// Looping var
-  int		progress;	// Progress so far...
-  int		error;		// Errors?
-  static char	message[1024];	// Progress message...
-  static int	installing = 0;	// Installing software?
+  int		i;			// Looping var
+  int		progress;		// Progress so far...
+  int		error;			// Errors?
+  static char	message[1024];		// Progress message...
+  static char	install_type[1024];	// EPM_INSTALL_TYPE env variable
+  static int	installing = 0;		// Installing software?
 
 
   Wizard->next();
@@ -795,13 +833,22 @@ next_cb(Fl_Button *, void *)
     if (NumInstTypes)
       PrevButton->activate();
 
+    // Figure out which type is chosen...
     for (i = 0; i < (int)(sizeof(TypeButton) / sizeof(TypeButton[0])); i ++)
       if (TypeButton[i]->value())
         break;
 
-    if (i < (int)(sizeof(TypeButton) / sizeof(TypeButton[0])) &&
-        InstTypes[i].num_products > 0)
-      Wizard->next();
+    if (i < (int)(sizeof(TypeButton) / sizeof(TypeButton[0])))
+    {
+      // Set the EPM_INSTALL_TYPE environment variable...
+      snprintf(install_type, sizeof(install_type), "EPM_INSTALL_TYPE=%s",
+               InstTypes[i].name);
+      putenv(install_type);
+
+      // Skip product selection if this type has a list already...
+      if (InstTypes[i].num_products > 0)
+        Wizard->next();
+    }
   }
 
   if (Wizard->value() == ConfirmPane)
@@ -861,18 +908,6 @@ next_cb(Fl_Button *, void *)
 }
 
 
-//
-// 'sort_dists()' - Compare two distribution names...
-//
-
-int				// O - Result of comparison
-sort_dists(const dist_t *d0,	// I - First distribution
-           const dist_t *d1)	// I - Second distribution
-{
-  return (strcmp(d0->name, d1->name));
-}
-
-
 //
 // 'type_cb()' - Handle selections in the type list.
 //
@@ -915,62 +950,11 @@ type_cb(Fl_Round_Button *w, void *)	// I - Radio button widget
 }
 
 
-//
-// 'log_cb()' - Add one or more lines of text to the installation log.
-//
-
-static void
-log_cb(int fd,			// I - Pipe to read from
-       int *fdptr)		// O - Pipe to read from
-{
-  int		bytes;		// Bytes read/to read
-  char		*bufptr;	// Pointer into buffer
-  static int	bufused = 0;	// Number of bytes used
-  static char	buffer[8193];	// Buffer
-
-
-  bytes = 8192 - bufused;
-  if ((bytes = read(fd, buffer + bufused, bytes)) <= 0)
-  {
-    // End of file; zero the FD to tell the install_dist() function to
-    // stop...
-
-    Fl::remove_fd(fd);
-    close(fd);
-    *fdptr = 0;
-
-    if (bufused > 0)
-    {
-      // Add remaining text...
-      buffer[bufused] = '\0';
-      InstallLog->add(buffer);
-      bufused = 0;
-    }
-  }
-  else
-  {
-    // Add bytes to the buffer, then add lines as needed...
-    bufused += bytes;
-    buffer[bufused] = '\0';
-
-    while ((bufptr = strchr(buffer, '\n')) != NULL)
-    {
-      *bufptr++ = '\0';
-      InstallLog->add(buffer);
-      strcpy(buffer, bufptr);
-      bufused -= bufptr - buffer;
-    }
-  }
-
-  InstallLog->bottomline(InstallLog->size());
-}
-
-
 //
 // 'update_size()' - Update the total +/- sizes of the installations.
 //
 
-static void
+void
 update_sizes(void)
 {
   int		i;		// Looping var
@@ -1062,7 +1046,6 @@ update_sizes(void)
 }
 
 
-
 //
-// End of "$Id: setup2.cxx,v 1.30 2002/06/04 19:17:31 mike Exp $".
+// End of "$Id: setup2.cxx 645 2005-10-14 18:01:51Z mike $".
 //
